//=============================================================================
// Copyright © 2016 FLIR Integrated Imaging Solutions, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of FLIR
// Integrated Imaging Solutions, Inc. ("Confidential Information"). You
// shall not disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with FLIR Integrated Imaging Solutions, Inc. (FLIR).
//
// FLIR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. FLIR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: fc2triclops.h 309688 2016-12-21 18:28:11Z silvanoa $
//=============================================================================

//=============================================================================
/** @file fc2triclops.h
 *  @brief Flycapture2triclops operations API
 */
//=============================================================================

#ifndef FC2TRICLOPS_H
#define FC2TRICLOPS_H

//=============================================================================
// PGR Includes
//=============================================================================

#include "flycapture2bridge.h"
#include <FlyCapture2.h>

//=============================================================================
// System Includes
//=============================================================================

#include <assert.h>

namespace Fc2Triclops {

/** @addtogroup FlyCapture2Bridge
 *
 * @{ */

/** @brief Point Grey's stereo cameras operating modes
 *
 *  Stereo cameras usually provide a dual image stream, interleaving the data from the right camera and the left
 *  camera (i.e. RLRLRL...), from the point of view of the camera, using Mono16 format for mono sensors and Raw16
 *  for color cameras.
 */
enum StereoCameraMode {
    TWO_CAMERA = 1,       /**< Dual stereo mode. When used with a triple-sensor stereo camera (e.g. BBXB3) it will be converted to TWO_CAMERA_NARROW */
    TWO_CAMERA_NARROW,    /**< Narrow (i.e. right and middle sensor) dual stereo mode */
    TWO_CAMERA_WIDE,      /**< Wide (i.e. right and left sensor) dual stereo mode */
};

/** @brief Sets a new stereo mode for a connected camera
 *
 * @param cam     The camera for which to set the new stereo mode
 * @param mode    The stereo mode to set
 *
 * @return An error code representing the outcome of the function
 * @retval ERRORTYPE_OK                     The function completed successfully
 * @retval ERRORTYPE_CONFIG_READ_FAILED     The retrieval of the configuration failed
 * @retval ERRORTYPE_NO_FMT7_SUPPORT        Format7 is not supported
 * @retval ERRORTYPE_INVALID_FMT7           Format7 configuration is invalid
 * @retval ERRORTYPE_NO_CORRECT_CAMERAMODE  The chosen stereo mode value is not correct for the camera provided
 */
ErrorType
setStereoMode(FlyCapture2::Camera &cam, StereoCameraMode &mode);

/** @brief Checks if a camera is a dual sensor Bumblebee camera
 *
 * @param cam     The camera to check
 * @param flag    Whether the provided camera is a dual sensor camera or not
 *
 * @return An error code representing the outcome of the function
 * @retval ERRORTYPE_OK                     The function completed successfully
 * @retval ERRORTYPE_CONFIG_READ_FAILED     The retrieval of the configuration failed
 *
 * @see isBBX3()
 */
ErrorType
isBB2(const FlyCapture2::Camera &cam, bool &flag);

/** @brief Checks if a camera is a triple sensor Bumblebee camera
 *
 * @param cam     The camera to check
 * @param flag    Whether the provided camera is a riple sensor camera or not
 *
 * @return An error code representing the outcome of the function
 * @retval ERRORTYPE_OK                     The function completed successfully
 * @retval ERRORTYPE_CONFIG_READ_FAILED     The retrieval of the configuration failed
 *
 * @see isBB2()
 */
ErrorType
isBBXB3(const FlyCapture2::Camera &cam, bool &flag);

/** @brief Handles FlyCapture 2 errors
 *
 * This function handles errors generated by FlyCapture 2 objects (e.g. camera, image ), while printing a formatted
 * log trace.
 *
 * @param fc2Error      The error to handle
 *
 * @return A FlyCapture2::ErrorType enumerator
 *
 * @see handleFc2TriclopsError()
 */
int
handleFc2Error(FlyCapture2::Error const &fc2Error);

/** @brief Handles FlyCapture2Bridge errors
 *
 * This function handles errors generated by FlyCapture2Bridge functions, while printing a formatted log trace.
 *
 * @param error         The error to handle
 * @param pCallNameStr  A string to add to the formatted log (e.g. the caller's name)
 *
 * @return ```(int)error + 1000```
 *
 * @see handleFc2TriclopsError()
 */
int
handleFc2TriclopsError(Fc2Triclops::ErrorType const &error,
                       const char *pCallNameStr);

/** @} */ // End of group FlyCapture2Bridge

} // end namespace Fc2Triclops

#endif // FC2TRICLOPS_H
