//=============================================================================
// Copyright © 2016 FLIR Integrated Imaging Solutions, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of FLIR
// Integrated Imaging Solutions, Inc. ("Confidential Information"). You
// shall not disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with FLIR Integrated Imaging Solutions, Inc. (FLIR).
//
// FLIR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. FLIR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: flycapture2bridge.h 309688 2016-12-21 18:28:11Z silvanoa $
//=============================================================================

//=============================================================================
/** @file flycapture2bridge.h
 *  @brief Flycapture2bridge operations API */
//=============================================================================

#ifndef FLYCAPTURE2BRIDGE_H
#define FLYCAPTURE2BRIDGE_H

//=============================================================================
// PGR Includes
//=============================================================================

#include "triclops.h"
#include <FlyCapture2.h>

namespace Fc2Triclops {

/** @defgroup FlyCapture2Bridge Point Grey's FlyCapture 2 SDK interfacing functions
 *
 * This group contains interfacing functions to manage side-by-side images retrieved from Point Grey cameras or to
 * directly control such cameras.
 */

/** @addtogroup FlyCapture2Bridge
 *
 * @{ */

/** @brief FlyCapture 2 interfacing error codes
 *
 * These errors are just a conversion of possible FlyCapture 2 errors.
 */
enum ErrorType {
    ERRORTYPE_OK,                        /**< No errors encountered */
    ERRORTYPE_CONFIG_READ_FAILED,        /**< Error while reading a camera configuration */
    ERRORTYPE_NO_CONFIG_FILE,            /**< Empty or non-existant configuration file */
    ERRORTYPE_NO_CAMERA,                 /**< Camera not connected */
    ERRORTYPE_CORRUPT_CONFIG_FILE,       /**< Corrupted configuration file */
    ERRORTYPE_FAILED_OUTPUT_ALLOCATION,  /**< Memory allocation failed */
    ERRORTYPE_FAILED_CONVERSION,         /**< Pixel format conversion failed */
    ERRORTYPE_INVALID_INPUT,             /**< One or more function parameters are invalid */
    ERRORTYPE_INVALID_OUTPUT,            /**< One or more output values are invalid due to failed operations */
    ERRORTYPE_NO_FMT7_SUPPORT,           /**< Format7 is not supported by the device */
    ERRORTYPE_INVALID_FMT7,              /**< Invalid Format7 request */
    ERRORTYPE_NO_CORRECT_CAMERAMODE      /**< Invalid camera mode requested */
};

/** @brief Generate a context from a connected camera
 *
 * @param cameraSerialNumber    The serial number of the connected camera
 * @param outputContext         The context to be filled
 *
 * @return An error code representing the outcome of the function
 * @retval ERRORTYPE_OK                     The function completed successfully
 * @retval ERRORTYPE_NO_CAMERA              The serial number provided refers to a camera that is not currently connected
 * @retval ERRORTYPE_CONFIG_READ_FAILED     The retrieval of the configuration failed
 * @retval ERRORTYPE_NO_CONFIG_FILE         The retrieved camera configuration is empty
 * @retval ERRORTYPE_CORRUPT_CONFIG_FILE    The retrieved camera configuration is corrupt
 *
 * @see triclopsGetDefaultContextFromFile()
 * @see triclopsGetDefaultContextFromMemory()
 */
ErrorType
getContextFromCamera(unsigned long cameraSerialNumber,
                     TriclopsContext *outputContext);

/** @brief Unpacks a raw, interleaved, stereo image into right and left raw images
 *
 * @param unprocessedImage      The interleaved raw image to unpack
 * @param isLittleEndian        Whether the image is encoded in little or big endian format.
 * @param unprocessedImageRight The unpacked right image
 * @param unprocessedImageLeft  The unpacked left image
 *
 * @return An error code representing the outcome of the function
 * @retval ERRORTYPE_OK                 The function completed successfully
 * @retval ERRORTYPE_FAILED_CONVERSION  The conversion failed
 * @retval ERRORTYPE_INVALID_INPUT      One or more input parameters are invalid
 * @retval ERRORTYPE_INVALID_OUTPUT     The function generated an invalid output
 *
 * @see unpackUnprocessedRgbImage()
 */
ErrorType
unpackUnprocessedRawOrMono16Image(const FlyCapture2::Image &unprocessedImage,
                                  bool isLittleEndian,
                                  FlyCapture2::Image &unprocessedImageRight,
                                  FlyCapture2::Image &unprocessedImageLeft);

/** @brief Unpacks a raw triple stereo image into right, left and middle raw images
 *
 *  This function unpacks triple stereo images that have been packed as the color channels of an RGB image. The
 *  triple stereo mode can be enabled on a connected camera using setStereoMode() to set THREE_CAMERA stereo mode.
 *
 * @param unprocessedImage          The packed raw image to unpack
 * @param unprocessedImageRight     The unpacked right image
 * @param unprocessedImageMiddle    The unpacked middle image
 * @param unprocessedImageLeft      The unpacked left image
 *
 * @return An error code representing the outcome of the function
 * @retval ERRORTYPE_OK                 The function completed successfully
 * @retval ERRORTYPE_FAILED_CONVERSION  The conversion failed
 * @retval ERRORTYPE_INVALID_INPUT      One or more input parameters are invalid
 * @retval ERRORTYPE_INVALID_OUTPUT     The function generated an invalid output
 *
 * @see unpackUnprocessedRawOrMono16Image()
 * @see setStereoMode()
 */
ErrorType
unpackUnprocessedRgbImage(const FlyCapture2::Image &unprocessedImage,
                          FlyCapture2::Image &unprocessedImageRight,
                          FlyCapture2::Image &unprocessedImageMiddle,
                          FlyCapture2::Image &unprocessedImageLeft);

/** @brief Generates a side-by-side image from a left and right image pair
 *
 * @param imageRgbRight     The right image
 * @param imageRgbLeft      The left image
 * @param packedRgbImage    The resulting side-by-side image
 *
 * @return An error code representing the outcome of the function
 * @retval ERRORTYPE_OK                 The function completed successfully
 * @retval ERRORTYPE_FAILED_CONVERSION  The conversion failed
 * @retval ERRORTYPE_INVALID_INPUT      One or more input parameters are invalid
 * @retval ERRORTYPE_INVALID_OUTPUT     The function generated an invalid output
 *
 * @see packThreeSideBySideRgbImage()
 */
ErrorType
packTwoSideBySideRgbImage(const FlyCapture2::Image &imageRgbRight,
                          const FlyCapture2::Image &imageRgbLeft,
                          FlyCapture2::Image &packedRgbImage);

/** @brief Generates a side-by-side image from a stereo image triplet
 *
 * @param imageRgbRight     The right image
 * @param imageRgbMiddle    The middle image
 * @param imageRgbLeft      The left image
 * @param packedRgbImage    The resulting side-by-side image
 *
 * @return An error code representing the outcome of the function
 * @retval ERRORTYPE_OK                 The function completed successfully
 * @retval ERRORTYPE_FAILED_CONVERSION  The conversion failed
 * @retval ERRORTYPE_INVALID_INPUT      One or more input parameters are invalid
 * @retval ERRORTYPE_INVALID_OUTPUT     The function generated an invalid output
 *
 * @see packTwoSideBySideRgbImage()
 */
ErrorType
packThreeSideBySideRgbImage(const FlyCapture2::Image &imageRgbRight,
                            const FlyCapture2::Image &imageRgbMiddle,
                            const FlyCapture2::Image &imageRgbLeft,
                            FlyCapture2::Image &packedRgbImage);

/** @} */ // End of group FlyCapture2Bridge
}


#endif // FLYCAPTURE2BRIDGE_H
