//=============================================================================
// Copyright © 2016 FLIR Integrated Imaging Solutions, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of FLIR
// Integrated Imaging Solutions, Inc. ("Confidential Information"). You
// shall not disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with FLIR Integrated Imaging Solutions, Inc. (FLIR).
//
// FLIR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. FLIR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: triclops3d.h 309688 2016-12-21 18:28:11Z silvanoa $
//=============================================================================

//=============================================================================
/** @file triclops3d.h
 *  @brief 3D operations API */
 //=============================================================================

#ifndef TRICLOPS3D_H
#define TRICLOPS3D_H

//=============================================================================
// Project Includes
//=============================================================================
#include <triclops.h>

#ifdef __cplusplus
extern "C"
{
#endif

//=============================================================================
// Types 
//=============================================================================
/** @addtogroup TriclopsTypes
 *
 * @{
 */ // start of group TriclopsTypes

/** @brief Three dimensional point structure
 *
 * @see TriclopsImage3d
 */
typedef struct TriclopsPoint3d {
    float point[3]; /**< The 3D coordinates in order [x, y, z] */
} TriclopsPoint3d;

/** @brief Three dimensional data structure
 *
 *  Structure used to represent a depth map using 3D coordinates with respect to the original pixel location.
 *  The rowinc parameter represents the length in bytes for each row; this value should be equal to
 *  ncols*sizeof(struct TriclopsPoint3d) unless any padding is being performed. The buffer pointer, if
 *  changed, should point to an initialised memory location large at least rowinc*nrows bytes.
 *
 * @see TriclopsPoint3d
 * @see triclopsExtractImage3d()
 * @see triclopsExtractWorldImage3d()
 */
typedef struct TriclopsImage3d {
    int nrows;
    /**< The number of rows in the image, representing the image width */
    int ncols;
    /**< The number of columns in the image, representing the image height */
    int rowinc;
    /**< The number of bytes for each row */
    TriclopsPoint3d *points;    /**< Pointer to the location of the image buffer */
} TriclopsImage3d;

/** @brief Quaternion structure
 *
 * This structure is used mainly to represent the Camera-to-world transform matrix, thus the name.
 *
 * @see TriclopsImage3d
 */
typedef struct {
    double matrix[4][4]; /**< Structure containing the values of the quaternion (i.e. matrix[0][0] = quaternion1,1;
                          *   matrix[4][4] = quaternion3,3) */
} TriclopsTransform;

/*@}*/ // End of group TriclopsTypes

/** @addtogroup Triclops3D
 *
 * @{ */

/** @brief Converts a 2D pixel coordinate in disparity space to a 3D point in camera coordinate system
 *
 * This function has been kept for retro-compatibility and is an alias for triclopsRCDFloatToXYZ().
 */
enum TriclopsError
triclopsRCDToXYZ(TriclopsContext context,
                 float row,
                 float col,
                 float disp,
                 float *x,
                 float *y,
                 float *z);
/** @brief Converts a 2D pixel coordinate in disparity space to a 3D point in camera coordinate system.
 *
 * This function uses a disparity value in floating point (i.e. already converted from the 12.4 fixed-point format
 * to its decimal value) and returns the coordinate of a point in 3D space.
 *
 * @param context The Triclops context used to compute disparities
 * @param row The y location of the input pixel
 * @param col The x location of the input pixel
 * @param disp The disparity value computed for the input pixel
 * @param x The resulting x coordinate of the pixel in camera coordinate system
 * @param y The resulting y coordinate of the pixel in camera coordinate system
 * @param z The resulting z coordinate of the pixel in camera coordinate system
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidCoordinates The provided 2D coordinates are invalid (i.e. less that zero or larger that the current outut resolution)
 * @retval TriclopsErrorInvalidDisparityValue The provided disparity is invalid (i.e. less or equal than 0 or plain invalid)
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsRCD16ToXYZ()
 */
enum TriclopsError
triclopsRCDFloatToXYZ(TriclopsContext context,
                      float row,
                      float col,
                      float disp,
                      float *x,
                      float *y,
                      float *z);
/** @brief Converts a 2D pixel coordinate in disparity space to a 3D point in camera coordinate system
 *
 * This function acts exactly like triclopsRCDFloatToXYZ() but takes a fixed point (i.e. 12.4 short integer) as
 * disparity value.
 */
enum TriclopsError
triclopsRCD16ToXYZ(TriclopsContext context,
                   int row,
                   int col,
                   unsigned short disp,
                   float *x,
                   float *y,
                   float *z);

/** @brief Converts a 3D point in camera coordinate system to a 2D pixel coordinate in disparity space.
 *
 * @param context The Triclops context used to compute disparities
 * @param x The x coordinate of the input pixel in camera coordinate system
 * @param y The y coordinate of the input pixel in camera coordinate system
 * @param z The z coordinate of the input pixel in camera coordinate system
 * @param row The resulting y coordinate of the input pixel in 2D space
 * @param col The resulting x coordinate of the input pixel in 2D space
 * @param disp The resulting disparity value computed for the input pixel
 *
 * Note: According to the input x and y coordinates, the corresponding col and row values could be negative 
 * or larger than the current output resolution, without generating any code error.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidDistanceValue The provided z coordinate is invalid (i.e. less or equal than 0)
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsRCDFloatToXYZ()
 * @see triclopsWorldXYZToRCD()
 */
enum TriclopsError
triclopsXYZToRCD(TriclopsContext context,
                 float x,
                 float y,
                 float z,
                 float *row,
                 float *col,
                 float *disp);
/** @brief Converts a 2D pixel coordinate in disparity space to a 3D point in world coordinate system.
 *
 * This function has been kept for retro-compatibility and is an alias for triclopsRCDFloatToWorldXYZ()
 */
enum TriclopsError
triclopsRCDToWorldXYZ(TriclopsContext context,
                      float row,
                      float col,
                      float disp,
                      float *x,
                      float *y,
                      float *z);

/** @brief Converts a 2D pixel coordinate in disparity space to a 3D point in world coordinate system
 *
 * This function computes the 3D point in camera coordinate system using triclopsRCDFloatToXYZ() and then applies the
 * rototranslation matrix (set using triclopsSetTriclopsToWorldTransform()) contained in the provided context to the
 * point.
 *
 * @param context The Triclops context used to compute disparities
 * @param row The y location of the input pixel
 * @param col The x location of the input pixel
 * @param disp The disparity value computed for the input pixel
 * @param x The resulting x coordinate of the pixel in world coordinate system
 * @param y The resulting y coordinate of the pixel in world coordinate system
 * @param z The resulting z coordinate of the pixel in world coordinate system
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidCoordinates The provided 2D coordinates are invalid
 * @retval TriclopsErrorInvalidDisparityValue The provided disparity value is invalid (i.e. less or equal than 0 or plain invalid)
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetTriclopsToWorldTransform()
 * @see triclopsRCD16ToWorldXYZ()
 */
enum TriclopsError
triclopsRCDFloatToWorldXYZ(TriclopsContext context,
                           float row,
                           float col,
                           float disp,
                           float *x,
                           float *y,
                           float *z);
/** @brief Converts a 2D pixel coordinate in disparity space to a 3D point in world coordinate system
 *
 * This function acts exactly like triclopsRCDFloatToXYZ() but takes a fixed point (i.e. 12.4 short integer) as
 * disparity value.
 */
enum TriclopsError
triclopsRCD16ToWorldXYZ(TriclopsContext context,
                        int row,
                        int col,
                        unsigned short disp,
                        float *x,
                        float *y,
                        float *z);
/** @brief Converts a 3D point in world coordinate system to a 2D pixel coordinate in disparity space.
 *
 * This function first applies an inverse rototranslation matrix (based on the one set in the provided context) and then
 * uses triclopsRCDToXYZ() to compute the 2D coordinate in disparity space.
 *
 * @param context The Triclops context used to compute disparities
 * @param x The x coordinate of the input pixel in camera coordinate system
 * @param y The y coordinate of the input pixel in camera coordinate system
 * @param z The z coordinate of the input pixel in camera coordinate system
 * @param row The resulting y coordinate of the input pixel in 2D space
 * @param col The resulting x coordinate of the input pixel in 2D space
 * @param disp The resulting disparity value computed for the input pixel
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidDistanceValue The provided z coordinate is invalid (i.e. less or equal than 0)
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsRCDFloatToXYZ()
 * @see triclopsWorldXYZToRCD()
 */
enum TriclopsError
triclopsWorldXYZToRCD(TriclopsContext context,
                      float x,
                      float y,
                      float z,
                      float *row,
                      float *col,
                      float *disp);
/** @brief Sets the 3D world coordinates rototranslation matrix to a TriclopsContext
 *
 * This function sets the provided homogeneous 4x4 rototranslation matrix (as the one presented in the detailed
 * description) as the world translation matrix for the provided triclops context. If the last row of the matrix
 * is not normalized (i.e. [0 0 0 1]) the function will try to normalize it. When providing such a matrix it is
 * advised to check the resulting matrix by calling triclopsGetTriclopsToWorldTransform().
 *
 * @param context The Triclops context to which set the matrix
 * @param transform The homogeneous 4x4 rototranslation matrix to set
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorSingularRotationMatrix The provided rototranslation matrix is singular
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsRCDFloatToWorldXYZ()
 * @see triclopsWorldXYZToRCD()
 * @see triclopsGetTriclopsToWorldTransform()
 */
enum TriclopsError
triclopsSetTriclopsToWorldTransform(TriclopsContext context,
                                    const TriclopsTransform *transform);
/** @brief Gets the 3D world coordinates rototranslation matrix to a TriclopsContext
 *
 * @param context The Triclops context that contains the matrix
 * @param transform The destination 4x4 rototranslation
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetTriclopsToWorldTransform()
 */
enum TriclopsError
triclopsGetTriclopsToWorldTransform(TriclopsContext context,
                                    TriclopsTransform *transform);

/** @brief Loads a 3D world coordinates rototranslation matrix from memory
 *
 * The file containing the rototranslation matrix needs to have the ".trans" extension, otherwise the function will
 * not recognise the file as valid.
 *
 * @param fileName The path, either relative or absolute, to the matrix file
 * @param transform The 4x4 matrix to fill with the data contained in the file
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorCorruptTransformFile The provided file path points to either an invalid transform
 *                                           file or to a file with the wrong extension.
 *
 * @see triclopsSetTriclopsToWorldTransform()
 */
enum TriclopsError
triclopsGetTransformFromFile(const char *fileName,
                             TriclopsTransform *transform);

/** @brief Saves a 3D world coordinates rototranslation matrix to memory
 *
 * The file containing the rototranslation matrix needs to have the ".trans" extension, otherwise the function will
 * not recognise the file as valid.
 *
 * @param fileName The path, either relative or absolute, to the matrix file
 * @param transform The 4x4 matrix to save to memory
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorCorruptTransformFile An error occurred while writing the matrix to memory
 *
 * @see triclopsSetTriclopsToWorldTransform()
 */
enum TriclopsError
triclopsWriteTransformToFile(const char *fileName,
                             const TriclopsTransform *transform);

/** @brief Initialises a 3D image structure
 *
 * The function initialises a TriclopsImage3d based on the output resolution set in the provided TriclopsContext. This
 * function is useful when trying to generate a full 3D point cloud based on the output of a depth map generated using
 * the provided context.
 *
 * @param context The context the 3D image is initialized for
 * @param ppimage The 3D image to initialize
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 *
 * @see triclopsDestroyImage3d()
 * @see triclopsExtractImage3d()
 */
enum TriclopsError
triclopsCreateImage3d(TriclopsContext context,
                      TriclopsImage3d **ppimage);

/** @brief Destroys a 3D image structure
 *
 * @param ppimage The 3D image to destroy
 */
void
triclopsDestroyImage3d(TriclopsImage3d **ppimage);

/** @brief Generates a 3D point cloud in camera space from the provided context
 *
 * The function generates a 3D point cloud in camera space using all the valid disparities contained in the disparity
 * buffer in the provided TriclopsContext. This function requires triclopsStereo() to be called at least once before
 * generating a 3D point cloud.
 *
 * Every invalid disparity will be given a 3D coordinate of [0, 0, 0].
 *
 * @param context The context where to retrieve the disparity map to convert
 * @param pimage The 3D image to fill with the generated point cloud
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsError3dImageSizeAndOutputResolutionMismatch The output resolution and the resolution of the 3D image are not the same
 * @retval TriclopsErrorDisparityMapAndOutputResolutionMismatch The output resolution and the resolution of teh disparity map are not the same
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsCreateImage3d()
 */
enum TriclopsError
triclopsExtractImage3d(TriclopsContext context,
                       TriclopsImage3d *pimage);

/** @brief Generates a 3D point in world space cloud from the provided context
 *
 * This function computes the 3D point cloud  in camera coordinate system using triclopsExtractImage3d() and then applies the
 * rototranslation matrix (set using triclopsSetTriclopsToWorldTransform()) contained in the provided context to the
 * point cloud.
 *
 * @param context The context where to retrieve the disparity map to convert
 * @param pimage The 3D image to fill with the generated point cloud
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsError3dImageSizeAndOutputResolutionMismatch The output resolution and the resolution of the 3D image are not the same
 * @retval TriclopsErrorDisparityMapAndOutputResolutionMismatch The output resolution and the resolution of teh disparity map are not the same
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsCreateImage3d()
 */
enum TriclopsError
triclopsExtractWorldImage3d(TriclopsContext context,
                            TriclopsImage3d *pimage);

/*@}*/ // End of group Triclops3D


/** @brief Returns the optical center for pinhole calculations.
 *  @ingroup CameraConfiguration
 *
 * It is important that the context already has the resolution set.
 * If triclopsSetResolution is not set, the returned value cannot be
 * used for calculations.  This image center can be used as the position
 * in the image plane of the optical center for pinhole camera calculations.
 *
 * @param context The context.
 * @param centerRow - A pointer that will contain the row position
 *              of the image center for the current resolution.
 * @param centerCol - A pointer that will contain the column position
 *              of the image center for the current resolution.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The provided context is invalid
 */
enum TriclopsError
triclopsGetImageCenter(const TriclopsContext context,
                       float *centerRow,
                       float *centerCol);

#ifdef __cplusplus
}
#endif

#endif  // #ifndef TRICLOPS3D_H
