//=============================================================================
// Copyright © 2016 FLIR Integrated Imaging Solutions, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of FLIR
// Integrated Imaging Solutions, Inc. ("Confidential Information"). You
// shall not disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with FLIR Integrated Imaging Solutions, Inc. (FLIR).
//
// FLIR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. FLIR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: triclopsbuffer.h 309688 2016-12-21 18:28:11Z silvanoa $
//=============================================================================

//=============================================================================
/** @file triclopsbuffer.h
 *  @brief External buffers operations API */
//=============================================================================

#ifndef TRICLOPSBUFFER_H
#define TRICLOPSBUFFER_H

//=============================================================================
// Project Includes
//=============================================================================
#include <triclops.h>

#ifdef __cplusplus
extern "C"
{
#endif

//=============================================================================
// Image Buffer Operations
//=============================================================================
//
// Group = Image Buffer Operations 

/** @defgroup ImageBuffersOperations Internal buffers manipulation functions
 *
 * Triclops provides functions to manage internal image buffers where to store 
 * images resulting from processing (i.e. rectified, edge and disparity images).
 * These buffers are an alternative to those managed internally by 
 * the context. It will be the user-s responsability to allocate sufficient memory for 
 * the buffer, and to de-allocate the buffer after it is no longer needed.
 */

/** @addtogroup ImageBuffersOperations
 * 
 * @{
 */ // start of group ImageBuffersOperations

/** @brief Sets the internal image buffer for the specified camera and image type 
 *  to be the buffer supplied by the user.
 *
 * This function allows the user to specify directly what memory he/she
 * wishes the output images to be deposited into. This memory will be used
 * by the stereo  kernel as working space.  This has the advantage of saving
 * a copy for tasks such as displaying to the screen.  The user may simply
 * set the output image buffer to his/her display buffer.  However, since
 * this memory will be used by the stereo kernel as working space, the
 * contents of the buffer may change with each call of triclopsRecify()
 * or triclopsStereo().  If the results are to be saved, it is the user's
 * responsibility to do so.  In addition, the user is responsible to allocate
 * sufficient memory for the buffer, and to de-allocate the buffer after
 * it is no longer needed. 
 * If the user has already called triclopsSetImageBuffer() for a particular
 * camera and image type, the stereo kernel will be using that buffer for
 * internal processing. 
 * If the user no longer wants to have the supplied buffer used by the stereo kernel, 
 * he/she can call triclopsUnsetImageBuffer() to inform
 * the stereo kernel that it is no longer available. 
 *
 * @param context The input context to set the buffer in
 * @param buffer A pointer to a user allocated buffer of sufficient size.
 * @param imageType The image type.
 * @param camera The camera to which associate the raw image originated.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidCamera  The requested camera is invalid or cannot be associated to this buffer type 
 * @retval TriclopsErrorInvalidImageType The requested image type is invalid or 
 *          buffers cannot be associated to the requested image type
 * @retval TriclopsErrorInvalidResolution The requested output resolution is invalid (i.e. either dimension is smaller than 1)
 *
 * @see triclopsGetImage()
 * @see triclopsUnsetImageBuffer()
 */
enum TriclopsError
triclopsSetImageBuffer(TriclopsContext context,
                       unsigned char *buffer,
                       TriclopsImageType imageType,
                       TriclopsCamera camera);

/** @brief Releases the user specified internal buffer for the specific camera 
 *   and image type
 *
 * If the user no longer wants to have the supplied buffer used by 
 * the stereo kernel, he/she can call this function to inform
 * the stereo kernel that it is no longer available. A new buffer will be
 * created for internal use, replacing the specified buffer.
 * The user should call this function before de-allocating the buffer.
 *
 * @param context The input context to be informed the specified buffer is no longer available.
 * @param imageType The image type.
 * @param camera The camera to which associate the raw image originated.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidCamera  The input camera is invalid
 * @retval TriclopsErrorInvalidImageType The requested image type is invalid or
 *          buffers cannot be associated to the requested image type
 * @retval TriclopsErrorInvalidResolution The requested output resolution is invalid (i.e. either dimension is smaller than 1)
 * @return TriclopsErrorBadMemoryAllocation The system returned an allocation error
 *          while requesting memory for the corresponding internal buffer
 *
 * @see triclopsGetImage()
 * @see triclopsSetImageBuffer()
 */
enum TriclopsError
triclopsUnsetImageBuffer(TriclopsContext context,
                         TriclopsImageType imageType,
                         TriclopsCamera camera);


/** @brief Sets the internal image buffer where 16-bit disparity images are written
 *
 * This function allows the user to set the location to which 16-bit 
 * (with subpixel interpolation) depth images are written to once they are processed.
 * Disparity images are always associated with the reference camera.
 * If the user has already called triclopsSetImage16Buffer() for a particular
 * camera and image type, the stereo kernel will be using that buffer for
 * internal processing. 
 * If the user no longer wants to have the supplied buffer used by the stereo kernel, 
 * he/she can call triclopsUnsetImage16Buffer() to inform the stereo kernel that 
 * it is no longer available. 
 *
 * @param context The context to set the 16-bit buffer in.
 * @param buffer A pointer to a user allocated buffer of sufficient size.
 * @param imageType The type of image to be written to the buffer.
 * @param camera The camera to write from.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidCamera  The requested camera is invalid or 
 *          is not equal to the reference image. 
 * @retval TriclopsErrorInvalidImageType The requested image type is invalid or
 *          cannot be associated to the 16-bit buffers.
 * @retval TriclopsErrorInvalidResolution The requested output resolution is invalid (i.e. either dimension is smaller than 1)
 *
 * @see triclopsGetImage()
 * @see triclopsUnsetImage16Buffer()
 */
enum TriclopsError
triclopsSetImage16Buffer(TriclopsContext context,
                         unsigned short *buffer,
                         TriclopsImage16Type imageType,
                         TriclopsCamera camera);

/** @brief Releases the 16-bit user internal buffer for the specific camera 
 *   and image type
 *
 * If the user no longer wants to have the supplied buffer used by 
 * the stereo kernel, he/she can call use this function to inform
 * the stereo kernel that it is no longer available. A new buffer will be
 * created for internal use, replacing the specified buffer.
 * The user should call this function before de-allocating the buffer.
 *
 * @param context The input context to be infomed the specified buffer is no longer available.
 * @param imageType The image type.
 * @param camera The camera 
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidCamera  The requested camera is invalid or
 *          is not equal to the reference image.
 * @retval TriclopsErrorInvalidImageType The requested image type is invalid or
 *          cannot be associated to the 16-bit buffers.
 * @retval TriclopsErrorInvalidResolution The requested output resolution 
 *          is invalid (i.e. either dimension is smaller than 1)
 *
 * @see triclopsGetImage()
 * @see triclopsSetImage16Buffer()
 */
enum TriclopsError
triclopsUnsetImage16Buffer(TriclopsContext context,
                           TriclopsImage16Type imageType,
                           TriclopsCamera camera);

/** @brief Allows the user to set the buffer to which the processed 32-bit color image
 * is written to.
 *
 * This function allows the user to set the location to which 32-bit color images 
 * are written to once they are processed. Color images are associated to 
 * Disparity images are always associated with the reference camera.
 * If the user has already called SetColorImageBuffer() for a particular
 * camera and image type, the stereo kernel will be using that buffer for
 * internal processing. 
 * If the user no longer wants to have the supplied buffer used by the stereo kernel,
 * he/she can call triclopsUnsetColorImageBuffer() to inform the stereo kernel that
 * it is no longer available. 
 *
 * @param context The context to set the 32-bit color buffer in.
 * @param camera The sensor for which to set the buffer
 * @param buffer A pointer to a user allocated buffer of sufficient size.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidCamera  The requested camera is invalid or 
 *          cannot be associated to this buffer type
 * @retval TriclopsErrorInvalidResolution The requested output resolution 
 *           is invalid (i.e. either dimension is smaller than 1)
 *
 * @see triclopsGetImage()
 * @see triclopsUnsetColorImageBuffer()
 */
enum TriclopsError
triclopsSetColorImageBuffer(TriclopsContext context,
                            TriclopsCamera camera,
                            TriclopsColorPixel *buffer);

/** @brief Releases the user specified internal color image buffer for the 
 *  specified camera. 
 *
 * If the user no longer wants to have the supplied buffer used by
 * the stereo kernel, he/she can call use this function to inform
 * the stereo kernel that it is no longer available. A new buffer will be
 * created for internal use, replacing the specified buffer.
 * The user should call this function before de-allocating the buffer.
 *
 * @param context The input context to be informed the specified buffer is no longer available.
 * @param camera The camera 
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidCamera  The requested camera is invalid or
 *          is not equal to the reference image.
 * @retval TriclopsErrorInvalidResolution The requested output resolution is 
 *          invalid (i.e. either dimension is smaller than 1)
 * @return TriclopsErrorBadMemoryAllocation The system returned an allocation error
 *          while requesting memory for the corresponding internal buffer
 *
 * @see triclopsGetImage()
 * @see triclopsSetColorImageBuffer()
 */

enum TriclopsError
triclopsUnsetColorImageBuffer(TriclopsContext context,
                              TriclopsCamera camera);

/** @} */ // End of Image Buffers Operations

#ifdef __cplusplus
}
#endif

#endif  // #ifndef TRICLOPSBUFFER_H
