//=============================================================================
// Copyright © 2016 FLIR Integrated Imaging Solutions, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of FLIR
// Integrated Imaging Solutions, Inc. ("Confidential Information"). You
// shall not disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with FLIR Integrated Imaging Solutions, Inc. (FLIR).
//
// FLIR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. FLIR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: triclopscontext.h 309688 2016-12-21 18:28:11Z silvanoa $
//=============================================================================

//=============================================================================
/** @file triclopscontext.h
 *  @brief Triclops Context API */
//=============================================================================

#ifndef TRICLOPSCONTEXT_H
#define TRICLOPSCONTEXT_H

//=============================================================================
//
// This file defines the API for TriclopsContext management functions
// within the Triclops Stereo Vision SDK.
//
//=============================================================================
#include "triclops.h"

#ifdef __cplusplus
extern "C"
{
#endif

//=============================================================================
// Function Prototypes  
//=============================================================================

//=============================================================================
// Triclops Context Manipulation
//=============================================================================

/** @defgroup TriclopsContext Triclops Context Manipulation
 *
 * Functions needed to manipulate the TriclopsContext structure
 */

/** @addtogroup TriclopsContext
 *
 * @{ */

/** @brief Sets up a new context using the information retrieved from a configuration file
 *
 * If the configuration did not end correctly the context pointer is set to NULL.
 *
 * @param defaultContext    The context to be initialized
 * @param filename          The path to the configuration file
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk                      The function completed successfully
 * @retval TriclopsErrorInvalidFilename         The provided filename is invalid (i.e. NULL pointer)
 * @retval TriclopsErrorBadMemoryAllocation     The operating system could not initialize a new context
 * @retval TriclopsErrorFileRead                The provided context is invalid
 * @retval TriclopsErrorCorruptConfigFile       The provided config file is invalid
 * @retval TriclopsErrorCameraConfigAlreadySet  The provided context has been already initialized
 * @retval TriclopsErrorInvalidContext          The function resulted in an invalid context
 *
 * @see triclopsGetDefaultContextFromMemory()
 * @see triclopsWriteDefaultContextToFile()
 * @see triclopsWriteCurrentContextToFile()
 */
enum TriclopsError
triclopsGetDefaultContextFromFile(TriclopsContext *defaultContext,
                                  const char *filename);

/** @brief Sets up a new context using the information retrieved from a configuration file loaded on memory
 *
 * This function performs the same operations as triclopsGetDefaultContextFromFile() but requires a pointer to the
 * configuration data loaded in memory, instead of a configuration file path.
 *
 * @param defaultContext    The context to be initialized
 * @param fileData          Pointer to the data loaded in memory
 *
 * @see triclopsGetDefaultContextFromFile()
 * @see triclopsWriteDefaultContextToFile()
 * @see triclopsWriteCurrentContextToFile()
 */
enum TriclopsError
triclopsGetDefaultContextFromMemory(TriclopsContext *defaultContext,
                                    void *fileData);

/** @brief Writes on file the default context configuration
 *
 * The configuration written on file by this function is the original configuration that would be retrieved from a
 * device or a device-specific configuration file.
 *
 * @param context           The context from which to retrieve the default configuration
 * @param filename          The path to the configuration file
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk                      The function completed successfully
 * @retval TriclopsErrorFileWrite               The system returned an error while trying to write the file on disk
 * @retval TriclopsErrorInvalidContext          The provided context is invalid
 *
 * @see triclopsGetDefaultContextFromFile()
 * @see triclopsWriteCurrentContextToFile()
 */
enum TriclopsError
triclopsWriteDefaultContextToFile(TriclopsContext context,
                                  const char *filename);

/** @brief Writes on file the current context configuration
 *
 * The configuration written on file by this function is the configuration that is currently in use by the context,
 * including additional configurations such as stereo and rectification parameters.
 *
 * @param context    The context from which to retrieve the default configuration
 * @param filename          The path to the configuration file
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk                      The function completed successfully
 * @retval TriclopsErrorFileWrite               The system returned an error while trying to write the file on disk
 * @retval TriclopsErrorInvalidContext          The provided context is invalid
 *
 * @see triclopsWriteDefaultContextToFile()
 */
enum TriclopsError
triclopsWriteCurrentContextToFile(TriclopsContext context,
                                  const char *filename);

/** @brief Copies the contents of a context to another context
 *
 * This function copies all the parameters and buffer pointers from the source context to the destination context. The
 * buffers will be decoupled once either context will change their contents.
 *
 * @param contextIn     The source context
 * @param contextOut    The destination context
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk                      The function completed successfully
 * @retval TriclopsErrorFileWrite               The system returned an error while trying to write the file on disk
 * @retval TriclopsErrorInvalidContext          The provided context is invalid
 *
 * @see triclopsDestroyContext()
 */
enum TriclopsError
triclopsCopyContext(const TriclopsContext contextIn,
                    TriclopsContext *contextOut);

/** @brief Destroys a context
 *
 * @param context     The context to destroy
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk                      The function completed successfully
 * @retval TriclopsErrorInvalidContext          The provided context is invalid
 */
enum TriclopsError
triclopsDestroyContext(TriclopsContext context);

/** @}*/ // End of group TriclopsContext

/**@addtogroup CameraConfiguration
 *
 * @{*/

/** @brief Retrieves the current camera configuration for the stereo pipeline
 *
 * @param context       The context from which to get the camera configuration
 * @param config        The current camera configuration
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk                      The function completed successfully
 * @retval TriclopsErrorInvalidContext          The provided context is invalid
 *
 * @see triclopsSetCameraConfiguration()
 * @see triclopsGetDeviceConfiguration()
 */
enum TriclopsError
triclopsGetCameraConfiguration(const TriclopsContext context,
                               TriclopsCameraConfiguration *config);

/** @brief Sets a new camera configuration for the stereo pipeline
 *
 * This function sets which pair of sensors are used for the current context, which affects both rectification, stereo
 * processing and world coordinates computation
 *
 * @param context       The context for which to set the camera configuration
 * @param config        The new camera configuration
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk                      The function completed successfully
 * @retval TriclopsErrorInvalidCameraConfig     The provided camera configuration is invalid (e.g. casted integer)
 * @retval TriclopsErrorCameraConfigNotSet      The provided camera configuration is invalid for the current context
 *                                              (e.g. wide baseline for a two-sensors camera)
 * @retval TriclopsErrorInvalidContext          The provided context is invalid
 *
 * @see triclopsGetDeviceConfiguration()
 * @see triclopsGetBaseline()
 */
enum TriclopsError
triclopsSetCameraConfiguration(const TriclopsContext context,
                               TriclopsCameraConfiguration config);

/** @brief Retrieves the current camera configuration for the stereo pipeline
 *
 * This is a legacy function. Now it is just a renaming of triclopsGetCameraConfiguration().
 *
 * @param context       The context from which to get the camera configuration
 * @param config        The current camera configuration
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk                      The function completed successfully
 * @retval TriclopsErrorInvalidContext          The provided context is invalid
 *
 * @see triclopsSetCameraConfiguration()
 */
enum TriclopsError
triclopsGetDeviceConfiguration(const TriclopsContext context,
                               TriclopsCameraConfiguration *config);

/** @brief Retrieves the serial number of the camera associated with a provided context
 *
 * @param context       The context from which to get the serial number
 * @param serialNumber  The serial number of the camera
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk                      The function completed successfully
 * @retval TriclopsErrorInvalidContext          The provided context is invalid
 */
enum TriclopsError
triclopsGetSerialNumber(const TriclopsContext context,
                        int *serialNumber);

/** @}*/ // End of group CameraConfiguration

#ifdef __cplusplus
}
#endif

#endif  // #ifndef TRICLOPSCONTEXT_H
