//=============================================================================
// Copyright © 2016 FLIR Integrated Imaging Solutions, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of FLIR
// Integrated Imaging Solutions, Inc. ("Confidential Information"). You
// shall not disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with FLIR Integrated Imaging Solutions, Inc. (FLIR).
//
// FLIR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. FLIR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: triclopsenum.h 309688 2016-12-21 18:28:11Z silvanoa $
//=============================================================================

//=============================================================================
/** @file triclopsenum.h
 *  @brief Triclops Enumerations */
//=============================================================================

#ifndef _TRICLOPS_ENUM_
#define _TRICLOPS_ENUM_

#include <triclopserror.h>

#ifdef __cplusplus
extern "C"
{
#endif

/** @defgroup TriclopsEnum Enumerators
 * Definitions of the enumerators used throughout the library
 */

/** @addtogroup TriclopsEnum
 *
 * @{ */ // start of group TriclopsEnum

/** @brief 8-bit per pixel image type enumerator
 *
 *      Enumerator used to define each specific 8-bit greyscale image buffer that can be publicly accessed
 *
 *  @see TriclopsImage16Type
 *  @see TriclopsColorImageType
 */
typedef enum TriclopsImageType {
    /** @brief Raw unrectified image
     *
     *      Image as retrieved from the camera or from a provided memory buffer. No correction for lens
     *      distortion or camera misalignment is applied on this buffer
     */
            TriImg_RAW = 0,
    /** @brief Rectified image
     *
     *      Image corrected for lens distortion and camera misalignment to fit a pinhole camera model
     */
            TriImg_RECTIFIED,
    /** @brief Edge image
     *
     * Image processed using a bandpass filter. Each pixel represent a biased derivative.
     * The value of the derivative can be computed by subtracting the bias (i.e. 128) from the value of each pixel
     */
            TriImg_EDGE
} TriclopsImageType;

/** @brief 16-bit per pixel image type enumerator
 *
 *      Enumerator used to define each specific 16-bit greyscale image buffer that can be publicly accessed
 *
 *  @see TriclopsImageType
 *  @see TriclopsColorImageType
 */
typedef enum TriclopsImage16Type {
    /** @brief Disparity Image
     *
     * Depth image represented using 12.4 fixed-point format disparities (i.e. 12 bits integer part, 4 bits
     * decimal part)
     */
            TriImg16_DISPARITY = 0

} TriclopsImage16Type;


/** @brief 32-bit per pixel image type enumerator
 *
 *      Enumerator used to define each specific 32-bit color image buffer that can be publicly accessed. The color images
 *      generated by Triclops use the RGBX format to represent colors.
 *
 *  @see TriclopsImageType
 *  @see TriclopsImage16Type
 *  @see TriclopsColorPixelFormat
 */
typedef enum TriclopsColorImageType {
    /** @brief Color non-rectified image
     *
     * Color image without corrections as was provided by the input source.
     */
            TriImg_RAW_COLOR = 0,

    /** @brief Rectified Color Image
     *
     * Image corrected for lens distortion and camera misalignment to fit a pinhole camera model, using a RAW
     * color image
     */
            TriImg_RECTIFIED_COLOR
} TriclopsColorImageType;

/** @brief Color pixel structure enumerator
 *
 *      Enumerator used to define how colors are represented in images. The color representations supported use either
 *      24 or 32 bits.
 *
 *  @see TriclopsColorImageType
 */
typedef enum {
    TriImg_Color_Pixel_Format_RGB = 0,  /**< 24-bit [Red; Green; Blue] color representation */
    TriImg_Color_Pixel_Format_BGR,      /**< 24-bit [Blue; Green; Red] color representation */
    TriImg_Color_Pixel_Format_RGBU,     /**< 32-bit [Red; Green; Blue; Unused] color representation */
    TriImg_Color_Pixel_Format_BGRU      /**< 32-bit [Blue; Green; Red; Unused] color representation */
} TriclopsColorPixelFormat;

/** @brief Sensor definition enumerator
 *
 *      Enumerator used to define the sensors available on Bumblebee products. In all of the cases TriCam_REFERENCE and
 *      TriCam_RIGHT refer to the same sensor, as Triclops' stereo algorithms use the feed from the right camera as
 *      reference for disparity computations
 *
 *      @see TriclopsCameraConfiguration
 */
typedef enum TriclopsCamera {
    TriCam_REFERENCE,
    TriCam_RIGHT,
    TriCam_LEFT,
} TriclopsCamera;

/** @brief Sensor configuration enumerator
 *
 *      Enumerator used to define which pair to use between the ones available on Bumblebee products. While Bumblebee 2 (BB2)
 *      have only two sensors, Bumblebee 3 (XB3) have three sensors. This is not to be confused with pan, which defines which
 *      of the two narrow configurations to use on XB3. TriCfg_2CAM_HORIZONTAL and TriCfg_2CAM_HORIZONTAL_NARROW
 *      represent the same sensor configuration on both BB2 and XB3, while TriCfg_2CAM_HORIZONTAL_WIDE is an invalid option
 *      for BB2.
 *
 *  @see TriclopsCamera
 */
typedef enum TriclopsCameraConfiguration {
    TriCfg_2CAM_HORIZONTAL = 0,         /**< Narrow (i.e. 12cm) baseline for BB2 or XB3 */
    TriCfg_2CAM_HORIZONTAL_NARROW = 0,  /**< Same as TriCfg_2CAM_HORIZONTAL */
    TriCfg_2CAM_HORIZONTAL_WIDE = 1,    /**< Wide (i.e. 24cm) baseline for XB3 */
    TriCfg_MAX_VALUE = 2
} TriclopsCameraConfiguration;

/** @brief Stereo Algorithm enumerator
 *
 *      Enumerator used to define the possible stereo algorithm that can be used
 *      in order to compute disparities between frames. OpenCV stereo algorithms can be
 *      used either in native mode (i.e. using the left frame as reference) or in
 *      Triclops mode (i.e. using the right frame as reference). Each supported OpenCV Semi-Global Block Matching
 *      implementation is represented in the OpenCVSGBMMode enumerator
 *
 *  @see OpenCVSGBMMode
 *  @see triclopsSetStereoAlgorithm()
 */
typedef enum {
    TriAlg_STANDARD,    /**< Triclops' stereo algorithm */
    TriAlg_OCVBM,       /**< OpenCV Block Matching (BM) algorithm */
    TriAlg_OCVBMLR,     /**< OpenCV BM algorithm, native mode */
    TriAlg_OCVSGBM,     /**< OpenCV Semi-Global Block Matching (SGBM) algorithm */
    TriAlg_OCVSGBMLR,    /**< OpenCV SGBM algorithm, native mode */
    TriAlg_LAST         /**< Placeholder for static code checks */
} TriclopsStereoAlgorithm;

/** @brief OpenCV Pre-filter type enumerator
 *
 *      Enumerator used to define the possible OpenCV Block Matching (BM) pre-filters used to compute the edge-filtered
 *      frames starting from the rectified frames fed to OpenCV BM.
 *
 *  @see triclopsSetOpenCVPreFilterCap()
 *  @see triclopsSetOpenCVBMPreFilterType()
 */
typedef enum {
    OCV_XSOBEL,                 /**< Normalized x-gradient Sobel operator */
    OCV_NORMALIZED_RESPONSE,    /**< Normalized response edge detection */
    OCV_TYPE_NOT_IMPLEMENTED    /**< Placeholder for yet-to-be supported pre-filters */
} OpenCVPrefilterType;

/** @brief OpenCV Semi-Global Block Matching (SGBM) implementation enumerator
 *
 *      Enumerator used to define the supported SGBM implementations that can be chosen from. SGBM has been implemented
 *      in three different ways. The standard implementation is a 5 directions approach aimed at computing comparable
 *      results without using as much space as the original 8 directions implementation. The 3 direction implementation
 *      is a newer approach which is optimized for parallel processing. The latter implementation is the one suggested
 *      in most cases when computing disparities using SGBM.
 *
 *  @see TriclopsStereoAlgorithm
 *  @see triclopsSetOpenCVSGBMMode()
 */
typedef enum {
    OCV_SGBM,                   /**< 5 directions implementation */
    OCV_HH,                     /**< Full-scale two-pass dynamic 8 directions implementation */
    OCV_SGBM_3WAY,              /**< 3 directions implementation optimized for parallel computation */
    OCV_MODE_NOT_IMPLEMENTED    /**< Placeholder for yet-to-be supported SGBM implementations */
} OpenCVSGBMMode;

/*@}*/  // end of group TriclopsEnum

#ifdef __cplusplus
}
#endif

#endif // _TRICLOPS_ENUM_
