//=============================================================================
// Copyright © 2016 FLIR Integrated Imaging Solutions, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of FLIR
// Integrated Imaging Solutions, Inc. ("Confidential Information"). You
// shall not disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with FLIR Integrated Imaging Solutions, Inc. (FLIR).
//
// FLIR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. FLIR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: triclopserror.h 309688 2016-12-21 18:28:11Z silvanoa $
//=============================================================================


//=============================================================================
/** @file triclopserror.h
 *  @brief Triclops Errors */
//=============================================================================

#ifndef _TRICLOPSERROR_
#define _TRICLOPSERROR_

#ifdef __cplusplus
extern "C"
{
#endif

/** @addtogroup TriclopsEnum
 *
 * @{ */ // start of group TriclopsEnum

/** @brief Error enumerator
 *
 *    Enumerator used by most of Triclops' functions to inform the user of the
 *    outcome of a function call. TriclopsError::TriclopsErrorOk is the enumerator
 *    valued used to indicate that the function completed the task correctly.
 */
enum TriclopsError {
    /** @brief No errors encountered
     *
     *  This value is returned whenever a called function executes correctly.
     */
            TriclopsErrorOk = 0,

//======================================================================================================================
// The "Invalid Parameters" section
//======================================================================================================================
    /** @brief One of the provided parameters was not valid
     *
     * This is returned when the parameters are outside range or, in the case of TriclopsEnum, not valid for the
     * enumerator (such as when casting integers to enum)
     */
            TriclopsErrorInvalidSetting,
    /** @brief The requested TriclopsCamera is not valid for the current context
     *
     * This error is returned when selecting a sensor not available in the provided context.
     */
            TriclopsErrorInvalidCamera,
    /** @brief The requested camera configuration is not valid */
            TriclopsErrorInvalidCameraConfig,
    /** @brief The requested Region of Interest is not valid */
            TriclopsErrorInvalidROI,
    /** An invalid parameter has been passed. */
            TriclopsErrorInvalidParameter,
    /** @brief The provided input resolution is invalid (i.e. either one or both dimensions are negative) */
            TriclopsErrorInvalidSourceResolution,
    /** @brief The chosen ROI and Output have different aspect ratios */
            TriclopsErrorRoiAndOutputAspectRatioMismatch,
    /** @brief The provided output resolution is invalid (i.e. either one or both dimensions are negative) */
            TriclopsErrorInvalidResolution,
    /** @brief The input coordinate (row or/and col) is invalid */
			TriclopsErrorInvalidCoordinates,
    /** @brief The standard ROI does not completely overlap with the input frames */
            TriclopsErrorSensorCalibratedAreaOutsideInput,
    /** @brief The provided row increment is not a multiple of the provided width */
            TriclopsErrorInvalidRowInc,
    /** @brief Invalid disparity value
    *
    * This is returned when the coordinates of a 3d point are requested
    * for a disparity value less than or equal to zero
    */
            TriclopsErrorInvalidDisparityValue,
    /** @brief Invalid distance value
    *
    * This is returned when the coordinates of a point of the disparity map
    * are requested for a distance value less than or equal to zero
    */
            TriclopsErrorInvalidDistanceValue,
    /** @brief The filename passed to the function is invalid */
            TriclopsErrorInvalidFilename,
    /** @brief An invalid number of disparities
    *
    * This is returned when number of disparities passed to
    * the Triclops stereo process is less than or equal to zero
    */
            TriclopsErrorInvalidNumberDisparities,
    /** @brief the stereo pair input is invalid
    *
    * This is returned when the size or the data buffer of one of the images
    * composing the stereo pair is invalid
    */
            TriclopsErrorInvalidStereoInput,
    /** @brief the processed image is neither a single or a side-by-side
    *
    * This might be returned when rectifying a raw color image. The rowinc values
    * must be four or eight time the number of columns
    */
            TriclopsErrorNeitherSingleNorSideBySideImage,
    /** @brief Passed an invalid pointer to a TriclopsVersion structure */
            TriclopsErrorInvalidVersion,

//======================================================================================================================
// The "Invalid Request" section
//======================================================================================================================
    /** @brief Error returned when a function is called without the proper setup
     *
     * This error is returned whenever the environment does not allow for the 
	 * execution of a function, e.g. when requiring a disparity map without 
	 * having called triclopsStereo or when rectifying/unrectifying a single pixel 
     * with rectification off */
            TriclopsErrorInvalidRequest,
    /** @brief The stereo algorithm requested is invalid or unknown */
            TriclopsErrorInvalidStereoAlgorithm,
    /** @brief Camera configuration is not set in the camera context */
            TriclopsErrorCameraConfigNotSet,
    /** @brief The function tried to set a camera configuration even though it was already set */
            TriclopsErrorCameraConfigAlreadySet,
    /** @brief The image type provided is either not valid for the function called or it does not exist */
            TriclopsErrorInvalidImageType,
    /** @brief The image format of the provided file is not supported */
            TriclopsErrorNotSupportedImageFileFormat,
    /** @brief The color pixel format of the provided file is not supported */
            TriclopsErrorNotSupportedColorPixelFormat,
    /** @brief The bitdepth of one or more provided images is either invalid for the function or not supported */
            TriclopsErrorInvalidBitdepthImage,
    /** @brief The requested width is larger than the supported width */
            TriclopsErrorExceedMaxNumberOutputColumns,
    /** @brief Rectification is not called yet
    *
    * This might be returned when a rectified image type is requested
    * before rectification ever called
    */
            TriclopsErrorRectificationNotCalled,
    /** @brief Edge correlation is not called yet
    *
    * This might be returned when an edge image type is requested
    * before edge processing ever called
    */
            TriclopsErrorEdgeCorrelationNotCalled,
    /** @brief Disparity computation is not called yet
    *
    * This might be returned when a disparity image type is requested
    * before disparity computation ever called
    */
            TriclopsErrorDisparityComputationNotCalled,
    /** @brief Failed the un-rectification of a rectified pixel */
            TriclopsErrorCannotUnrectifyPixel,

//======================================================================================================================
// The "Configuration file problems" section
//======================================================================================================================
    /** @brief The provided configuration file is non conformant with the specifications
     *
     * This error is returned whenever a provided filename points at a non-valid configuration file (e.g. missing
     * configuration fields, non-supported configuration version)
     */
            TriclopsErrorCorruptConfigFile,
    /** @brief The provided file contains invalid or contradictory configurations */
            TriclopsErrorInvalidConfig,
    /** @brief The provided context is either corrupt or invalid */
            TriclopsErrorInvalidContext,
    /** Can not find the transform file or its contents is invalid */
            TriclopsErrorCorruptTransformFile,
    /** @brief An error occurred when generating data for rectification */
            TriclopsErrorGenerationRectifyData,
    /** @brief The version of teh configuration file is invalid */
            TriclopsErrorInvalidConfigVersion,

//======================================================================================================================
// The "System Error" section
//======================================================================================================================


    /** @brief File read error */
            TriclopsErrorFileRead,
    /** @brief File write error */
            TriclopsErrorFileWrite,
    /** @brief The function encountered a memory allocation problem during execution */
            TriclopsErrorBadMemoryAllocation,
    /** @brief One or more of the image buffers used by the function called are not valid (i.e. null pointers) */
            TriclopsErrorInvalidBuffer,

//======================================================================================================================
// The "Miscellaneous" section
//======================================================================================================================
    /** @brief One or more internal parameters have been corrupted during the execution
     *
     * This might be returned when an user-specified output or a combination of internal parameters lead to a corruption
     * of internal settings, making impossible the correct execution of the function 
     */
            TriclopsErrorInvalidInternalSetting,
    /** @brief The size of the stereo input images mismatch */
            TriclopsErrorSizeStereoInputImagesMismatch,
    /** @brief Stereo input and output resolution differ
     *
     * This might be returned when rectification is off. The size of one image 
     * composing the stereo input differ from the requested output resolution 
     */
            TriclopsErrorStereoInputAndOutputResolutionMismatch,
    /** @brief Requested ROI and output resolution differ
    *
    * This might be returned when rectification is off and the processing on 
    * an ROI is turned on. The size of the ROI differ from the requested 
    * output resolution
    */
            TriclopsErrorRoiAndOutputResolutionMismatch,
    /** @brief Invalid stereo ROI to process
     *
     * The Region Of Interest, which the stereo process is carried out in, is invalid. 
     * This region is determined internally and results  
     * by subtracting mask size and minimum disparity. 
     */
            TriclopsErrorInvalidStereoROI,
    /** @brief 3d image size and output resolution differ */
            TriclopsError3dImageSizeAndOutputResolutionMismatch,
    /** @brief  Singular rotation matrix 
     * 
     * This might be returned when the rotation component of the trasform matrix 
     * contains singularities
     */
            TriclopsErrorSingularRotationMatrix,
    /** @brief Disparity map and output resolution differ */
            TriclopsErrorDisparityMapAndOutputResolutionMismatch,
    /** @brief The function encountered an unknown problem. */
            TriclopsErrorUnknown,
};

/** @} */ // end of group TriclopsEnum

/** @defgroup TriclopsDebug Debugging and Error Reporting */

/** @addtogroup TriclopsDebug
 * @{
 */

/** @brief Converts a Triclops error into a meaningful string
 *
 *  This function returns a string that describes the given enum TriclopsError.
 *  This allows error reporting software to report meaningful error messages
 *  to the user.
 *
 *  @param error The TriclopsError to convert
 *  @return A string with a full description of the provided TriclopsError
 *  @see TriclopsError
 */
char *
triclopsErrorToString(enum TriclopsError error);

/*@}*/ // end of group TriclopsDebug


#ifdef __cplusplus
}
#endif

#endif // _TRICLOPSERROR_H_
