//=============================================================================
// Copyright © 2016 FLIR Integrated Imaging Solutions, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of FLIR
// Integrated Imaging Solutions, Inc. ("Confidential Information"). You
// shall not disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with FLIR Integrated Imaging Solutions, Inc. (FLIR).
//
// FLIR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. FLIR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//
//=============================================================================
//=============================================================================
// $Id: triclopsimageio.h 309688 2016-12-21 18:28:11Z silvanoa $
//=============================================================================

//=============================================================================
/** @file triclopsimageio.h
 *   @brief Input/Output Image API */
//=============================================================================

#ifndef TRICLOPSIMAGEIO_H
#define TRICLOPSIMAGEIO_H

//=============================================================================
// Project Includes
//=============================================================================
#include "triclops.h"

#ifdef __cplusplus
extern "C"
{
#endif

//=============================================================================
// Image I/O Operations
//=============================================================================
// Group = Image I/O Operations

/** @defgroup TriclopsImage Image functions
 *
 * General image functions using to manipulate image structures used in Triclops. Triclops uses 8-bit greyscale images
 * as inputs for Edge filtering and disparity computations and accepts both 8-bit greyscale and 32-bit color images
 * for frame rectification. The images have to be packed in specific structures (i.e. TriclopsMonoStereoPair and
 * TriclopsColorStereoPair) in order to be accepted as valid stereo inputs.
 *
 * Depth maps are stored as 16-bit images, with every pixel representing a depth in fixed point format (12 integer
 * bits, 4 decimal bits).
 *
 * ## Triclops' Image management
 *
 * All of the functions that retrieve frames from contexts perform only shallow copies of the frames, effectively
 * pointing at internal data locations. This means that it is necessary to perform a hard copy of the image data every
 * time the frame data is needed after destroying the context or after refreshing a context's internal buffers.
 *
 * The file formats currently supported are PGM, PPM, PNG and TIFF formats.
 */

/** @addtogroup TriclopsImage
*
* @{ */

/** @brief Retrieves an 8-bit greyscale image form a context
 *
 * The function retrieves an image from the internal frame buffers of the provided context. The frame can be relative to
 * either sensor of the camera.
 *
 * @param context The context where to retrieve the frame
 * @param imageType The frame to retrieve
 * @param camera The sensor relative to the frame
 * @param image The image to fill with the data retrieved
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidCamera The chosen sensor is non-existent (e.g. int casted to TriclopsCamera)
 * @retval TriclopsErrorInvalidRequest The chosen frame is TriImg_EDGE even though Edge filtering is deactivated
 * @retval TriclopsErrorInvalidParameter The chosen frame is non-existent (e.g. int casted to TriclopsCamera)
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see TriclopsImage
 * @see triclopsSaveImage()
 * @see triclopsGetImage16()
 */
enum TriclopsError
triclopsGetImage(const TriclopsContext context,
                 TriclopsImageType imageType,
                 TriclopsCamera camera,
                 TriclopsImage *image);
/** @brief Joins two greyscale images in a single side-by-side structure
 *
 * This function performs only a shallow copy of the two images provided, meaning that both images should be valid
 * for the whole life of the stereo pair structure. The two images need to have the same resolution for this function
 * to correctly execute.
 *
 * @param context Any context
 * @param right The right image of the pair
 * @param left The left image of the pair
 * @param triclopsInput The stereo pair to populate using the images
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidBuffers Either one or both input images pointers are NULL
 * @retval TriclopsErrorInvalidSourceResolution The resolutions of the input images are different
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsBuildMonoStereoPairFromFile()
 * @see triclopsBuildColorStereoPairFromBuffers()
 * @see triclopsFreeMonoStereoPair()
 */
enum TriclopsError
triclopsBuildMonoStereoPairFromBuffers(const TriclopsContext context,
                                       TriclopsImage *right,
                                       TriclopsImage *left,
                                       TriclopsMonoStereoPair *triclopsInput);

/** @brief Loads a greyscale picture from file in a side-by-side structure
 *
 * @param context Any context
 * @param filename Path to the side-by-side picture
 * @param triclopsInput The stereo pair to populate using the image from memory
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidBuffers The image load function returned a NULL pointer [This should never happen]
 * @retval TriclopsErrorInvalidSourceResolution One of the two dimensions of the input image is invalid (e.g. too large)
 * @retval TriclopsErrorNotSupportedImageFileFormat The format of the provided image is not supported
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidBitdepthImage The size of one pixel in terms of number of bits is invalid.
 *
 * @see triclopsBuildMonoStereoPairFromBuffers()
 * @see triclopsBuildColorStereoPairFromFile()
 * @see triclopsFreeMonoStereoPair()
 */
enum TriclopsError
triclopsBuildMonoStereoPairFromFile(const TriclopsContext context,
                                    const char *filename,
                                    TriclopsMonoStereoPair *triclopsInput);
/** @brief Joins two color images in a single side-by-side structure
 *
 * This function performs the same operation than triclopsBuildMonoStereoPairFromBuffers() but it is intended for
 * color images
 *
 * @param context Any context
 * @param right The right image of the pair
 * @param left The left image of the pair
 * @param triclopsInput The stereo pair to populate using the images
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidBuffers Either one or both input images pointers are NULL
 * @retval TriclopsErrorInvalidSourceResolution The resolutions of the input images are different
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsBuildColorStereoPairFromFile()
 * @see triclopsBuildMonoStereoPairFromBuffers()
 * @see triclopsFreeColorStereoPair()
 */
enum TriclopsError
triclopsBuildColorStereoPairFromBuffers(const TriclopsContext context,
                                        TriclopsColorImage *right,
                                        TriclopsColorImage *left,
                                        TriclopsColorStereoPair *triclopsInput);

/** @brief Loads a color picture from file in a side-by-side structure
 *
 * This function performs the same operation than triclopsBuildMonoStereoPairFromFile() but it is intended for
 * color images. 
 *
 * @param context Any context
 * @param filename Path to the side-by-side picture
 * @param pixelFormat Pixel color encoding of the picture to load
 * @param triclopsInput The stereo pair to populate using the image from memory
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidBuffers The image load function returned a NULL pointer [This should never happen]
 * @retval TriclopsErrorInvalidSourceResolution One of the two dimensions of the input image is invalid (e.g. too large)
 * @retval TriclopsErrorNotSupportedImageFileFormat The format of the provided image is not supported
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidBitdepthImage The size of one pixel in terms of number of bits is invalid.

 * @see triclopsBuildColorStereoPairFromBuffers()
 * @see triclopsBuildMonoStereoPairFromFile()
 * @see triclopsFreeColorStereoPair()
 */
enum TriclopsError
triclopsBuildColorStereoPairFromFile(const TriclopsContext context,
                                     const char *filename,
                                     TriclopsColorPixelFormat pixelFormat,
                                     TriclopsColorStereoPair *triclopsInput);

/** @brief Releases a greyscale side-by-side image structure
 *
 * This function is useful when a side-by-side image loaded using triclopsBuildMonoStereoPairFromFile() is not needed
 * anymore (e.g. at exit). This function will have side effects when called on a TriclopsMonoStereoPair created using
 * triclopsBuildMonoStereoPairFromBuffers() as in this case the structure uses two shallow copies of the images
 * provided at construction, resulting in the destruction of the two original images.
 *
 * @param stereoPair The greyscale side-by-side structure to free
 * @param isSingleBuffer Set to `true` if the structure has only the right buffer set
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 *
 * @see triclopsBuildMonoStereoPairFromBuffers()
 * @see triclopsBuildMonoStereoPairFromFile()
 */
enum TriclopsError
triclopsFreeMonoStereoPair(TriclopsMonoStereoPair *stereoPair,
                           bool isSingleBuffer);

/** @brief Releases a color side-by-side image structure
 *
 * This function performs the same operation as triclopsFreeMonoStereoPair().
 *
 * @param stereoPair The color side-by-side structure to free
 * @param isSingleBuffer Set to `true` if the structure has only the right buffer set
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 *
 * @see triclopsBuildColorStereoPairFromBuffers()
 * @see triclopsBuildColorStereoPairFromFile()
 */
enum TriclopsError
triclopsFreeColorStereoPair(TriclopsColorStereoPair *stereoPair,
                            bool isSingleBuffer);

/** @brief Retrieves a 16-bit greyscale image form a context
 *
 * The function performs the same operation as triclopsGetImage(), but only retrieves 16-bit frames (i.e. depth
 * maps).
 *
 * @param context The context where to retrieve the frame
 * @param imageType The frame to retrieve
 * @param camera The sensor relative to the frame
 * @param image The image to fill with the data retrieved
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidCamera The chosen sensor is non-existent (e.g. int casted to TriclopsCamera)
 * @retval TriclopsErrorInvalidParameter The chosen frame is non-existent (e.g. int casted to TriclopsCamera)
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see TriclopsImage16
 * @see triclopsSaveImage16()
 * @see triclopsGetImage()
 */
enum TriclopsError
triclopsGetImage16(const TriclopsContext context,
                   TriclopsImage16Type imageType,
                   TriclopsCamera camera,
                   TriclopsImage16 *image);

/** @brief Retrieves a 32-bit color image form a context
*
* The function performs the same operation as triclopsGetImage(), but retrieves color frames (i.e. colored unrectified
* and rectified images)
*
* @param context The context where to retrieve the frame
* @param imageType The frame to retrieve
* @param camera The sensor relative to the frame
* @param image The image to fill with the data retrieved
*
* @return An error code representing the outcome of the function
* @retval TriclopsErrorOk The function completed successfully
* @retval TriclopsErrorInvalidCamera The chosen sensor is non-existent (e.g. int casted to TriclopsCamera)
* @retval TriclopsErrorInvalidParameter The chosen frame is non-existent (e.g. int casted to TriclopsCamera)
* @retval TriclopsErrorInvalidContext The provided context is invalid
*
* @see TriclopsColorImage
* @see triclopsSaveColorImage()
* @see triclopsGetImage()
*/
enum TriclopsError
triclopsGetColorImage(const TriclopsContext context,
                      TriclopsColorImageType imageType,
                      TriclopsCamera camera,
                      TriclopsColorImage *image);

/** @brief Saves a greyscale image to memory
*
* The path should contain a full filename, including the extension, in order for the function to understand which
* file format to use when saving the image. 
*
* @param image The image to save to memory
* @param filename The path where to save the image
*
* @return An error code representing the outcome of the function
* @retval TriclopsErrorOk The function completed successfully
* @retval TriclopsErrorInvalidParameter The provided path is an empty string
* @retval TriclopsErrorNotSupportedImageFileFormat The chosen image format is not supported
*
* @see triclopsSaveImage16()
* @see triclopsSaveColorImage()
*/
enum TriclopsError
triclopsSaveImage(TriclopsImage *image,
                  const char *filename);

/** @brief Saves a 16-bit image to memory
 *
 * The path should contain a full filename, including the extension, in order for the function to understand which
 * file format to use when saving the image. 
 *
 * @param image The image to save to memory
 * @param filename The path where to save the image
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidParameter The provided path is an empty string
 * @retval TriclopsErrorNotSupportedImageFileFormat The chosen image format is not supported
 *
 * @see triclopsSaveImage()
 * @see triclopsSaveColorImage()
 */
enum TriclopsError
triclopsSaveImage16(TriclopsImage16 *image,
                    const char *filename);

/** @brief Saves a color image to memory
*
* This function saves color frames (i.e. colored unrectified and rectified images)
*
* The path should contain a full filename, including the extension, in order for the function to understand which
* file format to use when saving the image.
*
* @param image The image to save to memory
* @param filename The path where to save the image
* @param pixelFormat The format of the color pixel
*
* @return An error code representing the outcome of the function
* @retval TriclopsErrorOk The function completed successfully
* @retval TriclopsErrorInvalidFilename The provided file path is invalid
* @retval TriclopsErrorNotSupportedImageFileFormat The chosen image format is not supported
* @retval TriclopsErrorNotSupportedColorPixelFormat The format of pixels is not supported
*
* @see triclopsSaveImage()
* @see triclopsSaveImage16()
*/
enum TriclopsError
triclopsSaveColorImage(TriclopsColorImage *image,
                       const char *filename,
                       TriclopsColorPixelFormat pixelFormat);

/** @brief Load a 8bit image from a file 
 *
 * The function performs the same operation as triclopsLoadImage16FromFile(), 
 * triclopsLoadColorImageFromFile(), but only retrieves 8-bit frames 
 * (i.e. raw, rectified and edge images).
 * 
 * The path should contain a full filename, including the extension, in order for the function to understand which
 * file format to use when loading the image.
 *
 * @param filename The path to the image filename. 
 * @param image    The pointer to the structure describing the loaded image
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorFileRead The image has not been successfully from file
 * @retval TriclopsErrorNotSupportedImageFileFormat The image format is not supported or unknown
 * @retval TriclopsErrorInvalidBitdepthImage The size of one pixel in bits is not 8, as expected
 *
 * @see triclopsLoadImage16FromFile()
 * @see triclopsLoadColorImageFromFile()
 */
enum TriclopsError
triclopsLoadImageFromFile(const char *filename, TriclopsImage *image);

/** @brief Load a 8-bit image from a buffer
 *
 * The function performs the same operation as triclopsLoadImageFromFile(),
 * but only retrieves 8-bit frames from a buffer rather than a file. 
 * It carries out a shallow copy of the buffer.
 *
 * @param buffer The pointer to the buffer containing image values
 * @param nrows The number of rows of the buffer
 * @param ncols The number of columns of the buffer
 * @param rowinc The pitch, or row increment for the image. rowinc must be a multiple of cols
 * @param image The pointer to the structure describing the image 
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidBuffer The buffer is null
 * @retval TriclopsErrorInvalidRowInc The row increment is invalid. 
 *
 * @see triclopsLoadImage16FromBuffer()
 * @see triclopsLoadColorImageFromBuffer()
 * @see triclopsLoadImageFromFile()
 */
enum TriclopsError
triclopsLoadImageFromBuffer(unsigned char *buffer,
                            unsigned int nrows,
                            unsigned int ncols,
                            unsigned int rowinc,
                            TriclopsImage *image);

/** @brief Load a 16-bit image from a file
*
* The function performs the same operation as triclopsLoadImageFromFile(),
* triclopsLoadColorImageFromFile(), but only retrieves 16-bit frames
* (i.e. disparity maps).
*
* The path should contain a full filename, including the extension, in order for the function to understand which
*
* @param filename The path to the image filename.
* @param image    The pointer to the structure describing the loaded image
*
* @return An error code representing the outcome of the function
* @retval TriclopsErrorOk The function completed successfully
* @retval TriclopsErrorFileRead The image has not been successfully from file
* @retval TriclopsErrorNotSupportedImageFileFormat The image format is not supported or unknown
* @retval TriclopsErrorInvalidBitdepthImage The size of one pixel in bits is not 16, as expected
*
* @see triclopsLoadImageFromFile()
* @see triclopsLoadColorImageFromFile()
*/
enum TriclopsError
triclopsLoadImage16FromFile(const char *filename, TriclopsImage16 *image);

/** @brief Load a 16-bit image from a buffer
 *
 * The function performs the same operation as triclopsLoadImage16FromFile(),
 * but only retrieves 16-bit frames from a buffer rather than a file.
 * It carries out a shallow copy of the buffer.
 *
 * @param buffer The pointer to the buffer containing image values
 * @param nrows The number of rows of the buffer
 * @param ncols The number of columns of the buffer
 * @param rowinc The pitch, or row increment for the image. rowinc must be a multiple of cols
 * @param image The pointer to the structure describing the image
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidBuffer The buffer is null
 * @retval TriclopsErrorInvalidRowInc The row increment is invalid.
 *
 * @see triclopsLoadImageFromBuffer()
 * @see triclopsLoadColorImageFromBuffer()
 * @see triclopsLoadImage16FromFile()
 */
enum TriclopsError
triclopsLoadImage16FromBuffer(unsigned short *buffer,
                              unsigned int nrows,
                              unsigned int ncols,
                              unsigned int rowinc,
                              TriclopsImage16 *image);

/** @brief Load a color image from a file
 *
 * The function performs the same operation as triclopsLoadImageFromFile(),
 * triclopsLoadImage16FromFile(), but only retrieves color frames
 * (i.e. raw and rectified images).
 *
 * The path should contain a full filename, including the extension, in order for the function to understand which
 *
 * @param filename    The path to the image filename
 * @param pixelFormat The format of the color pixel
 * @param image       The pointer to the structure describing the loaded image
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorFileRead The image has not been successfully from file
 * @retval TriclopsErrorNotSupportedImageFileFormat The image format is not supported or unknown
 * @retval TriclopsErrorInvalidBitdepthImage The size of one pixel in bits is invalid. Supported pixel size values are 24 and 32.
 * @retval TriclopsErrorNotSupportedColorPixelFormat The format of pixels is not supported
 *
 * @see triclopsLoadImageFromFile()
 * @see triclopsLoadImage16FromFile()
 */
enum TriclopsError
triclopsLoadColorImageFromFile(const char *filename,
                               TriclopsColorPixelFormat pixelFormat,
                               TriclopsColorImage *image);

/** @brief Load a color image from a buffer
 *
 * The function performs the same operation as triclopsLoadColorImageFromFile(),
 * but only retrieves a color frames from a buffer rather than a file.
 * It carries out a shallow copy of the buffer.
 *
 * @param buffer The pointer to the buffer containing image values
 * @param nrows The number of rows of the buffer
 * @param ncols The number of columns of the buffer
 * @param rowinc The pitch, or row increment for the image. rowinc must be a multiple of cols
 * @param image The pointer to the structure describing the image
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidBuffer The buffer is null
 * @retval TriclopsErrorInvalidRowInc The row increment is invalid.
 *
 * @see triclopsLoadImageFromBuffer()
 * @see triclopsLoadImage16FromBuffer()
 * @see triclopsLoadColorImageFromFile()
 */
enum TriclopsError
triclopsLoadColorImageFromBuffer(TriclopsColorPixel *buffer,
                                 unsigned int nrows,
                                 unsigned int ncols,
                                 unsigned int rowinc,
                                 TriclopsColorImage *image);

/*@}*/ // End of group TriclopsImage

#ifdef __cplusplus
}
#endif

#endif  // #ifndef TRICLOPSIMAGEIO_H
