//=============================================================================
// Copyright © 2016 FLIR Integrated Imaging Solutions, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of FLIR
// Integrated Imaging Solutions, Inc. ("Confidential Information"). You
// shall not disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with FLIR Integrated Imaging Solutions, Inc. (FLIR).
//
// FLIR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. FLIR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: triclopsrectify.h 309688 2016-12-21 18:28:11Z silvanoa $
//=============================================================================

//=============================================================================
/** @file triclopsrectify.h
 *  @brief Rectification API */
//=============================================================================

#ifndef TRICLOPSRECTIFY_H
#define TRICLOPSRECTIFY_H

//=============================================================================
// Project Includes
//=============================================================================
#include <triclops.h>

#ifdef __cplusplus
extern "C"
{
#endif

//=============================================================================
// Rectification and Camera Geometry
//=============================================================================
// Group = Rectification Configuration

/**@defgroup RectificationConfiguration Rectification configuration functions
 *
 * Triclops provides functions to prepare to and control image rectification 
 * using a calibrated camera. These functions are configured and controlled 
 * through specific parameters. 
 */

/**@addtogroup RectificationConfiguration
 *
 * @{
 */  // start of group RectificationConfiguration

/** @brief Sets the resolution of the raw images that the library will be processing later.
 *
 * This function sets the expected resolution of the raw images.
 * This function is provided primarily to support feature based stereo application
 * where one is expected to make direct calls to triclopsRectifyPixel() and
 * triclopsUnrectifyPixel() on a point by point basis.  For regular stereo
 * application where an entire image will be rectified every time, the application
 * should use triclopsPrepareRectificationData() which in addition to setting
 * up both the source and rectification resolution, it also creates the rectification
 * table to speed up the full image rectification calls.
 *
 * @param context  The context
 * @param nSrcRows The number of rows in the raw images.
 * @param nSrcCols The number of columns in the raw images.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The input context is invalid.
 * @retval TriclopsInvalidSourceResolution The number of rows or columns is a negative number.
 *
 * @see triclopsGetSourceResolution()
 * @see triclopsRectifyPixel()
 * @see triclopsUnrectifyPixel()
 * @see triclopsPrepareRectificationData()
 */
enum TriclopsError
triclopsSetSourceResolution(TriclopsContext context,
                            int nSrcRows,
                            int nSrcCols);

/** @brief Sets the region of interest
 *
 * This function sets the region of interest (ROI) where rectification is carried out.
 * One ROI is drawn in the rectified space for each calibrated lens. The resolution of this calibrated space 
 * is equal to the source resolution.
 * ROI processing is not enabled calling this function, but triclopsSetRoiOn() has to be called.
 *
 * @param context  The input context.
 * @param roi The input region of interest.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The input context is invalid.
 * @retval TriclopsErrorInvalidSetting The provided ROI has one or more invalid internal values.
 *
 * @see triclopsGetRoi()
 * @see triclopsSetRoiOn()
 */
enum TriclopsError
triclopsSetRoi(TriclopsContext context,
               TriclopsROI roi);

/** @brief Enables rectification in a region of interest
 *
 * This function enables rectification in a region of interest (ROI)
 * rather than in the entire image. The ROI is defined calling triclopsSetRoi().
 *
 * @param context  The input context.
 * @param roiOn A Boolean indicating whether rectification in a ROI should be turned on or off. 
 *
 * @return An error code representing the outcome of the function.
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The input context is invalid.
 *
 * @see triclopsSetRoi()
 * @see triclopsGetRoiOn()
 */
enum TriclopsError
triclopsSetRoiOn(TriclopsContext context,
                 TriclopsBool roiOn);

/** @brief Sets the resolution of the resultant images.  This includes rectified, 
 *  disparity and edge images.
 *
 * @param context The input context.
 * @param nrows   The number of rows in the output images.
 * @param ncols   The number of columns in the output images.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The input context is invalid.
 * @retval TriclopsErrorInvalidResolution The resolution is composed of one or more negative numbers.
 * @retval TriclopsErrorExceedMaxNumberOutputColumns The number of columns exceeds the maximum limit.
 *
 * @see triclopsGetResolution()
 */
enum TriclopsError
triclopsSetResolution(TriclopsContext context,
                      int nrows,
                      int ncols);

/** @brief Sets the resolution of the resultant images and prepare data for rectrification.
 *
 *  This function sets the desired resolution of the output images and also
 *  immediately constructs the rectification tables.  For large images, the
 *  construction of the rectification can take a while. This function allows
 *  you to control when the construction takes place, otherwise it will occur
 *  during the first call to triclopsRectify().  The resolution of the input
 *  images must be specified at this time, as this is necessary for the
 *  construction of the tables. The output images include the rectified,
 *  disparity and edge images. 
 *  For feature based stereo application where rectification of the entire image
 *  is not needed, one should call triclopsSetResolution() and triclopsSetSourceResolution()
 *  only (these are much simpler functions), and then simply proceeds to call
 *  triclopsRectifyPixel() and triclopsUnrectifyPixel() for the small set of feature
 *  pixels needed.
 *
 * @param context The input context.
 * @param nrows   The number of rows in the output images.
 * @param ncols   The number of columns in the output images.
 * @param nInputRows  The number of rows in the input images.
 * @param nInputCols  The number of columns in the input images.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The provided context is either invalid or 
 *          contains one or more invalid rectification warpers
 * @retval TriclopsErrorInvalidSourceResolution The resolution of the provided source is invalid (i.e. either dimension is smaller than 1)
 * @retval TriclopsErrorInvalidResolution The requested output resolution is invalid (i.e. either dimension is smaller than 1)
 * @retval TriclopsErrorExceedMaxNumberOutputColumns The number of columns for the output resolution exceeds the maximum limit
 * @retval TriclopsErrorInvalidROI The ROI requested is either invalid or is not completely overlapped with the source image
 * @retval TriclopsErrorRoiAndOutputAspectRatioMismatch The aspect ratio of the requested ROI and 
            the requested output are different
 * @retval TriclopsErrorInvalidInternalSetting One of the internal settings used for the generation of the rectification
 * 			warpers is not valid.
 * @retval TriclopsErrorBadMemoryAllocation The system returned an allocation error while creating one of the
 *          internal buffers (i.e. Edge or Rectified buffers) or while requesting more memory for rectification warpers
 * @retval TriclopsErrorGenerationRectifyData An error occurred when generating data for rectification 
 *
 * @see triclopsSetResolution()
 * @see triclopsSetSourceResolution(),
 * @see triclopsGetResolution()
 * @see triclopsGetSourceResolution()
 * @see triclopsSetRectify()
 */
enum TriclopsError
triclopsPrepareRectificationData(TriclopsContext context,
                                 int nrows,
                                 int ncols,
                                 int nInputRows,
                                 int nInputCols);

/** @brief Retrieves the current resolution for raw images
 *
 * @param context The context.
 * @param nSrcRows A pointer to the retrieved number of rows in the raw images.
 * @param nSrcCols A pointer to the retrieved number of columns in the raw images.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The input context is invalid.
 *
 * @see triclopsSetSourceResolution()
 * @see triclopsGetResolution()
 */
enum TriclopsError
triclopsGetSourceResolution(const TriclopsContext context,
                            int *nSrcRows,
                            int *nSrcCols);

/** @brief Retrieves the output resolution
 *
 * This function retrieves the resolution of the resultant images.  This includes rectified, 
 * disparity and edge images.
 *
 * @param context The context.
 * @param nrows A pointer to the retrieved number of rows in the output images.
 * @param ncols A pointer to the retrieved number of columns in the output images.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The input context is invalid.
 *
 * @see triclopsSetResolution()
 * @see triclopsGetSourceResolution()
 */
enum TriclopsError
triclopsGetResolution(const TriclopsContext context,
                      int *nrows,
                      int *ncols);

/** @brief Retrieves the region of interest
 *
 * This function retrives the region of interest (ROI) where rectification is carried out.
 *
 * @param context  The input context.
 * @param roi A pointer to a ROI structure that will store the current region of interest.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The input context is invalid.
 * @retval TriclopsErrorInvalidSetting One of the elements composing
 * the ROI structure is negative or equal to zero.
 *
 * @see triclopsSetRoi()
 */
enum TriclopsError
triclopsGetRoi(TriclopsContext context,
               TriclopsROI *roi);

/** @brief Retrieves the state of the rectification in a region of interest
 *
 * @param context The input context.
 * @param roiOn A pointer to a Boolean that will store the current setting.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The input context is invalid.
 *
 * @see triclopsSetRoiOn()
 */
enum TriclopsError
triclopsGetRoiOn(TriclopsContext context,
                 TriclopsBool *roiOn);


/** @} */ // End of RectificationConfiguration

//=============================================================================
// Rectification
//=============================================================================
//
// Group = Rectification

/** @addtogroup Rectification
 *
 *@{
 */

/** @brief Turns rectification on or off
 *
 * @param context The input context.
 * @param on A Boolean indicating whether rectification should be turned on or off.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The input context is invalid.
 *
 * @see triclopsGetRectify()
 */
enum TriclopsError
triclopsSetRectify(TriclopsContext context,
                   TriclopsBool on);

/** @brief Retrieves the state of the rectification feature.
 *
 * @param context The context.
 * @param on A pointer to a Boolean that will store the current setting..
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The input context is invalid.
 *
 * @see triclopsSetRectify()
 */
enum TriclopsError
triclopsGetRectify(const TriclopsContext context,
                   TriclopsBool *on);

/** @brief Rectifies a raw color image
 *
 * This function will rectify a 32-bit color image corresponding to a raw image
 * composing the color side-by-side structure of the stereo input. 
 * It is useful for creating rectified color images for display.
 *
 * @param context The context where to retrieve the processing settings and 
 *                to rectify the requested raw color image 
 * @param camera The camera from which the raw image originated
 * @param input A pointer to the side-by-side color stereo input to process
 * @param output A pointer to the rectified color image
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The provided context is either invalid or 
 *                                contains one or more invalid rectification warpers
 * @retval TriclopsErrorInvalidCamera The chosen camera is invalid
 * @retval TriclopsErrorInvalidStereoInput The provided side-by-side structure has one or
*           more invalid internal values
 * @retval TriclopsErrorSensorCalibratedAreaOutsideInput The sensor calibrated area is not completely overlapped with the source image
 * @retval TriclopsErrorSizeStereoInputImagesMismatch The two images contained in the provided side-by-side structure have different resolutions
 * @retval TriclopsErrorInvalidSourceResolution The resolution of the provided source is invalid (i.e. either dimension is smaller than 1)
 * @retval TriclopsErrorInvalidResolution The requested output resolution is invalid (i.e. either dimension is smaller than 1)
 * @retval TriclopsErrorInvalidROI The ROI requested has one or more invalid internal values or 
 *          is not completely overlapped with the source image
 * @retval TriclopsErrorRoiAndOutputAspectRatioMismatch The aspect ratio of the requested ROI and 
 *          the requested output are different
 * @retval TriclopsErrorInvalidInternalSetting An internal invalid setting occurred
 * @retval TriclopsErrorNeitherSingleNorSideBySideImage The raw image relative to the selected camera is 
 *          neither a single nor a side-by-side image
 *
 * @see TriclopsColorStereoPair
 * @see TriclopsColorImage
 * @see triclopsColorRectify()
 */
enum TriclopsError
triclopsRectifyColorImage(TriclopsContext context,
                          TriclopsCamera camera,
                          TriclopsColorStereoPair *input,
                          TriclopsColorImage *output);


/** @brief Converts a pixel location from the unrectified image to that of the rectified image
 *
 * This function converts coordinates of a pixel location from the unrectified image  
 * to those of the rectified image.
 * The source and output image dimensions must have been previously set either via a call
 * to triclopsPrepareRectificationData() or via a call to triclopsSetSourceResolution followed by triclopsSetResolution.
 * Note that if ROI processing is enabled, both input and output pixel locations refer to the ROI coordinate system 
 *
 * @param context The context where to retrieve the processing settings and to convert pixel coordinayes
 * @param camera  The camera for which the pixel should be rectified
 * @param rowIn   The row coordinate of the location of the pixel to rectify.
 * @param colIn   The column coordinate of the location of the pixel to rectify.
 * @param rowOut  A pointer to the row coordinate of the location of the rectified pixel.
 * @param colOut  A pointer to the column coordinate of the location of the rectified pixel.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The provided context is either invalid or contains one or 
 *          more invalid rectification warpers
 * @retval TriclopsErrorInvalidCamera The chosen camera is invalid
 * @retval TriclopsErrorInvalidSourceResolution The resolution of the provided source is invalid (i.e. either dimension is smaller than 1)
 * @retval TriclopsErrorBadMemoryAllocation The system returned an allocation error 
 *          while requesting memory for rectification warp functions or warpers
 * @retval TriclopsErrorInvalidCoordinates The input pixel coordinates are invalid.
 *
 * @see triclopsRectify()
 * @see triclopsUnrectifyPixel()
 * @see triclopsSetSourceResolution()
 * @see triclopsSetResolution()
 * @see triclopsPrepareRectificationData()
 */
enum TriclopsError
triclopsRectifyPixel(const TriclopsContext context,
                     TriclopsCamera camera,
                     float rowIn,
                     float colIn,
                     float *rowOut,
                     float *colOut);

/** @brief Converts a pixel location from the rectified image to that of the unrectified image
 *
 * This function converts coordinates of a pixel location from the rectified image
 * to those of the unrectified image coordinates.
 *
 * This version will accommodate input rectified pixel locations that are
 * outside of the normal rectified image bounds, as long as the corresponding
 * unrectified location is within its own image bounds.
 * The source and output image dimensions must have been previously set either via a call
 * to triclopsPrepareRectificationData() or via a call to triclopsSetSourceResolution followed by triclopsSetResolution.
 * Note that if ROI processing is enabled, both input and output pixel locations refer to the ROI coordinate system
 *
 * @param context The context where to retrieve the processing settings and to convert pixel coordinayes
 * @param camera  The camera for which the pixel should be unrectified
 * @param rowIn   The row coordinate of the location of the pixel to unrectify.
 * @param colIn   The column coordinate of the location of the pixel to unrectify.
 * @param rowOut  A pointer to the row coordinate of the location of the unrectified pixel.
 * @param colOut  A pointer to the column coordinate of the location of the unrectified pixel.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The provided context is either invalid or 
            contains one or more invalid rectification warpers
 * @retval TriclopsErrorInvalidCamera The chosen camera is invalid
 * @retval TriclopsErrorInvalidSourceResolution The resolution of the provided source is invalid (i.e. either dimension is smaller than 1)
 * @retval TriclopsErrorBadMemoryAllocation The system returned an allocation error 
 *          while requesting memory for rectification warp functions or warpers
 * @retval TriclopsErrorInvalidCoordinates The input pixel coordinates are invalid.
 *
 * @see triclopsRectifyPixel()
 * @see triclopsSetSourceResolution()
 * @see triclopsSetResolution()
 * @see triclopsPrepareRectificationData()
 */
enum TriclopsError
triclopsUnrectifyPixel(const TriclopsContext context,
                       TriclopsCamera camera,
                       float rowIn,
                       float colIn,
                       float *rowOut,
                       float *colOut);

/** @} */ // End of Rectification

//=============================================================================
// Configuration
//=============================================================================
//
// Group = CameraConfiguration 

/**@defgroup CameraConfiguration Camera configuration functions */

/**@addtogroup CameraConfiguration 
 *
 * @{
 */

/** @brief Retrieves the baseline of the cameras.
 *
 * This function retrieves the baseline of the cameras in meters. The
 * context must have already been read.
 *
 * @param context The context.
 * @param baseline A pointer to the retrieved value of the baseline in meters.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The provided context is invalid
 *
 * @see triclopsGetDefaultContextFromFile()
 */
enum TriclopsError
triclopsGetBaseline(const TriclopsContext context,
                    float *baseline);

/** @brief Retrieves the focal length of the cameras
 *
 * This function returns the focal length of the system. The focal length
 * is in 'pixels' for the current selected output resolution.  All cameras'
 * rectified images have the same focal length.  The default context
 * must have been read before this call can be made.
 *
 * @param context The context.
 * @param focallength A pointer to the retrieved value of the focallength in pixels.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The provided context is invalid
 *
 * @see triclopsGetDefaultContextFromFile()
 * @see triclopsSetResolution()
 */
enum TriclopsError
triclopsGetFocalLength(const TriclopsContext context,
                       float *focallength);

/** @} */ // End of CameraConfiguration

/**@defgroup AspectRatio Aspect ratio functions
 *
 * Output images can have an arbitrary aspect ratio. The output size and aspect ratio are requested
 * by setting the output resolution ( \see triclopsSetResolution() ). 
 * However, cameras are calibrated with a fixed aspect ratio. Therefore rectification is carried out
 * by using an internal image with the same aspect ratio as that of the calibrated 
 * camera. The size of the internal image is determined to contain centrally the requested output size.
 * An output image structure, with the requested aspect ratio, will be constructed by shallow copy 
 * from the internal image.
 * 
*/

/**@addtogroup AspectRatio
*
* @{
*/

/** @brief Retrieves the aspect ratio of the calibrated camera.
 *
 * This function retrieves the aspect ratio ( the number of columns to rows
 * ratio) of the calibrated camera. 
 *
 * @param context The context which to retrieve the aspect ratio value from.
 * @param aspectRatio A pointer to the location containing the retrieved value of the aspect ratio.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The provided context is invalid
 *
 */
enum TriclopsError
triclopsGetCalAspectRatio(const TriclopsContext context,
                          float *aspectRatio);

/** @brief Determines the optimal number of rows.
 *
 * This function determines the optimal number of rectified rows as a function of the
 * expected number of columns of the rectified image and the aspect ratio of the calibrated camera.
 *
 * @param context The context which to retrieve the aspect ratio from.
 * @param ncols The chosen width of the resulting image
 * @param nrows The height computed using the camera's optimal aspect ratio and the given width
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The provided context is invalid
 *
 * @see triclopsGetCalAspectRatio()
 * @see triclopsGetOptimalNumColumns()
 *
 */
enum TriclopsError
triclopsGetOptimalNumRows(const TriclopsContext context,
                          int ncols,
                          int *nrows);

/** @brief Determines the optimal number of columns.
 *
 * This function determines the optimal number of rectified cols as a function of the
 * expected number of rows of the rectified image and the aspect ratio of the calibrated camera.
 *
 * @param context The context which to retrieve the aspect ratio value from.
 * @param nrows The chosen height of the resulting image
 * @param ncols The width computed using the camera's optimal aspect ratio and the given height
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully.
 * @retval TriclopsInvalidContext The provided context is invalid
 *
 * @see triclopsGetCalAspectRatio()
 * @see triclopsGetOptimalNumRows()
 *
 */
enum TriclopsError
triclopsGetOptimalNumColumns(const TriclopsContext context,
                             int nrows,
                             int *ncols);

/** @} */ // End of AspectRatio
#ifdef __cplusplus
}
#endif

#endif  // #ifndef TRICLOPSRECTIFY_H
