//=============================================================================
// Copyright © 2016 FLIR Integrated Imaging Solutions, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of FLIR
// Integrated Imaging Solutions, Inc. ("Confidential Information"). You
// shall not disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with FLIR Integrated Imaging Solutions, Inc. (FLIR).
//
// FLIR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. FLIR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: triclopsstereo.h 309688 2016-12-21 18:28:11Z silvanoa $
//=============================================================================

//=============================================================================
/** @file triclopsstereo.h
 *  @brief Stereo API */
//=============================================================================

#ifndef TRICLOPSSTEREO_H
#define TRICLOPSSTEREO_H

#include <triclops.h>

#ifdef __cplusplus
extern "C"
{
#endif
//=============================================================================
// Function Prototypes
//=============================================================================

/** @addtogroup Rectification
 *
 * @{
 */  // start of group Rectification

/** @brief Prepares the internal image buffers for depth computation
 *
 * All the resulting processed frames are stored in the provided
 * context. The processed frames can be retrieved using triclopsGetImage().
 *
 * This function has to be called before triclopsStereo otherwise that function will return an error code.
 *
 * @param context The context where to retrieve the processing settings and where to store the results
 * @param input The side-by-side structure to process
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidStereoInput The provided side-by-side structure has one or more invalid internal values
 * @retval TriclopsErrorSizeStereoInputImagesMismatch The two images contained in the provided side-by-side structure have different resolutions
 * @retval TriclopsErrorSensorCalibratedAreaOutsideInput The sensor calibrated area is not completely overlapped with the source image
 * @retval TriclopsErrorInvalidSourceResolution The resolution of the provided source is invalid (i.e. either dimension is smaller than 1)
 * @retval TriclopsErrorInvalidResolution The requested output resolution is invalid (i.e. either dimension is smaller than 1)
 * @retval TriclopsErrorInvalidROI The ROI requested is either invalid or is not completely overlapped with the source image
 * @retval TriclopsErrorRoiAndOutputAspectRatioMismatch The aspect ratio of the requested ROI and the requested output are different
 * @retval TriclopsErrorStereoInputAndOutputResolutionMismatch The resolution of the frames forming the side-by-side
 *          image and the output resolution are different. This error is returned only when rectification has been deactivated.
 * @retval TriclopsErrorRoiAndOutputResolutionMismatch The resolution of the frames forming the side-by-side
 *          image and the selected ROI are different. This error is returned only when rectification has been deactivated.
 * @retval TriclopsErrorBadMemoryAllocation The system returned an allocation error while creating one of the
 *          internal buffers (i.e. Edge or Rectified buffers) or while requesting more memory for rectification warpers
 * @retval TriclopsErrorInvalidInternalSetting One of the internal settings used for the generation of the rectification
 * 			warpers is not valid.
 * @retval TriclopsErrorGenerationRectifyData The library was not able to generate one or more rectification warpers.
 * @retval TriclopsErrorInvalidContext The provided context is either invalid or contains one or more invalid
 *          rectification warpers.
 *
 * @see TriclopsMonoStereoPair
 * @see triclopsColorRectify()
 * @see triclopsBuildMonoStereoPairFromBuffers()
 * @see triclopsBuildMonoStereoPairFromFile()
 * @see triclopsGetImage()
 * @see triclopsSetEdgeCorrelation()
 * @see triclopsSetRectify()
 * @see triclopsSetRoi()
 * @see triclopsStereo()
 */
enum TriclopsError
triclopsRectify(TriclopsContext context,
                TriclopsMonoStereoPair *input);

/** @brief Prepares the internal color image buffers for depth computation
 *
 * This function performs the same operations as triclopsRectify(), but processes colored images.
 * Rectified color buffers are internally converted to monochrome before depth computation.
 *
 * @param context The context where to retrieve the processing settings and where to store the results
 * @param input The side-by-side color structure to process
 *
 * @see TriclopsMonoStereoPair
 * @see triclopsColorRectify()
 * @see triclopsBuildColorStereoPairFromBuffers()
 * @see triclopsBuildColorStereoPairFromFile()
 * @see triclopsGetImage()
 * @see triclopsGetColorImage()
 * @see triclopsSetEdgeCorrelation()
 * @see triclopsSetEdgeMask()
 * @see triclopsSetRoi()
 * @see triclopsSetRectify()
 * @see triclopsStereo()
 */
enum TriclopsError
triclopsColorRectify(TriclopsContext context,
                     TriclopsColorStereoPair *input);

/** @brief Switches edge pre-processing on or off
 *
 * Edge pre-processing is required for texture validation.
 *
 * @param context   The context for which to enable/disable edge pre-processing
 * @param on        Whether edge-preprocessing should be on or off
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsGetEdgeCorrelation()
 * @see triclopsSetTextureValidation()
 */
enum TriclopsError
triclopsSetEdgeCorrelation(TriclopsContext context,
                           TriclopsBool on);

/** @brief Retrieves the current edge pre-processing status for a context
 *
 * @param context   The context from which to get the status of edge pre-processing
 * @param on        The retrieved status
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetEdgeCorrelation()
 */
enum TriclopsError
triclopsGetEdgeCorrelation(const TriclopsContext context,
                           TriclopsBool *on);

/** @brief Sets a new edge pre-processing mask size
 *
 * @param context   The context for which to set the edge mask size
 * @param masksize  The new mask size, which needs to be an odd number between 3 and 11
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidSetting The provided mask size is invalid
 *
 * @see triclopsGetEdgeMask()
 */
enum TriclopsError
triclopsSetEdgeMask(TriclopsContext context,
                    int masksize);

/** @brief Retrieves the current edge pre-processing mask size
 *
 * @param context   The context from which to set the edge mask size
 * @param masksize  The retrieved mask size
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetEdgeMask()
 */
enum TriclopsError
triclopsGetEdgeMask(const TriclopsContext context,
                    int *masksize);

/** @} */ // End of Rectification

/** @addtogroup StereoProcessing
 *
 * @{
 */  // start of group StereoProcessing

/** @brief Performs depth computation and pixel validations
 *
 * This function performs stereo processing and validations based on the parameters set in the provided context. The
 * resulting depth map is stored inside the provided context and can be retrieved using triclopsGetImage16().
 *
 * This function returns an error if neither triclopsRectify() nor triclopsColorRectify() has been previously
 * called.
 *
 * @param context The context on which to perform depth computation
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorRectificationNotProcessed This function has been called before calling triclopsRectify() or triclopsColorRectify()
 * @retval TriclopsErrorInvalidCameraConfig The stereo algorithm set in the provided context is non-existant (i.e. wrong triclopsSetStereoAlgorithm() parameter)
 * @retval TriclopsErrorInvalidStereoROI The ROI requested, combined with stereo and edge mask, is invalid (e.g. the
 *          resulting area is too small)
 * @retval TriclopsErrorUnknown The stereo process has encountered a problem during computation and failed. This error is returned only when selecting
 *          OpenCV for stereo computations.
 *
 * @see triclopsGetImage()
 * @see triclopsSetStereoMask()
 * @see triclopsSetDoStereo()
 */
enum TriclopsError
triclopsStereo(TriclopsContext context);

/** @brief Switches stereo computation on or off
 *
 * triclopsStereo() will return TriclopsErrorOk without doing nothing if
 * stereo computation is set to off
 *
 * @param context   The context for which to enable/disable stereo computation
 * @param on        Whether stereo computation should be on or off
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsGetDoStereo()
 */
enum TriclopsError
triclopsSetDoStereo(TriclopsContext context,
                    TriclopsBool on);

/** @brief Retrieves the current stereo computation status for a context
 *
 * @param context   The context from which to get the status of stereo computation
 * @param on        The retrieved status
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetDoStereo()
 */
enum TriclopsError
triclopsGetDoStereo(const TriclopsContext context,
                    TriclopsBool *on);

/** @brief Sets a new stereo disparity range
 *
 * The maximum disparity range that can be set is currently 4078.
 *
 * @param context       The context for which to set the new disparity range
 * @param minDisparity  The minimum disparity to compute
 * @param maxDisparity  How many disparities to compute
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidNumberDisparities The provided number of disparities is invalid (i.e. less or equal than zero)
 * @retval TriclopsErrorInvalidSetting The combination of minDisparity and maxDisparity is invalid (e.g. negative
 *          minDisparity, maximum computed disparity too big, needed offset is too big).
 *
 * @see triclopsGetDisparity()
 * @see triclopsSetDoStereo()
 */

//  FIXME: This function should have a numDisparities parameter instead of maxDisparity, as per implementation
//  FIXME: Lots of functions check whether input parameters are smaller than zero, should we use UNSIGNED INT instead of INT?
enum TriclopsError
triclopsSetDisparity(TriclopsContext context,
                     int minDisparity,
                     int maxDisparity);


/** @brief Retrieves the current stereo disparity range
 *
 * @param context       The context from which to get the disparity range
 * @param minDisparity  The minimum disparity to compute
 * @param maxDisparity  How many disparities to compute
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetDisparity()
 */

//  FIXME: This function name is misleading, in my opinion.
//          While the setter sets min and num, this retrieves the minimum and maximum. Also the name seems to hint that
//          this function computes the disparity map and retrieves it.
enum TriclopsError
triclopsGetDisparity(const TriclopsContext context,
                     int *minDisparity,
                     int *maxDisparity);

/** @brief Sets a new stereo mask size
 *
 * @param context   The context for which to set the stereo mask size
 * @param masksize  The new mask size, which needs to be an odd number between 1 and 23
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidSetting The provided mask size is invalid
 *
 * @see triclopsGetStereoMask()
 */
enum TriclopsError
triclopsSetStereoMask(TriclopsContext context,
                      int masksize);

/** @brief Sets a new stereo mask size ignoring the limits set in the library
 *
 * This function allows the user to set an arbitrary stereo mask. This can lead to a number of unpredictable
 * behaviours, such as overflowing or random disparities.
 *
 * @param context   The context for which to set the stereo mask size
 * @param size      The new mask size, which needs to be an odd number
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidSetting The provided mask size is invalid
 *
 * @see triclopsGetStereoMask()
 */
enum TriclopsError
triclopsSetAnyStereoMask(TriclopsContext context,
                         int size);

/** @brief Retrieves the current stereo mask size
 *
 * @param context   The context from which to set the stereo mask size
 * @param size      The retrieved mask size
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetStereoMask()
 */
enum TriclopsError
triclopsGetStereoMask(const TriclopsContext context,
                      int *size);

/** @brief Sets a new stereo algorithm
 *
 * @param context   The context for which to set the stereo algorithm
 * @param algorithm The new stereo algorithm to use
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsGetStereoAlgorithm()
 */
enum TriclopsError triclopsSetStereoAlgorithm(const TriclopsContext context,
                                              TriclopsStereoAlgorithm algorithm);

/** @brief Retrieves the current stereo algorithm
 *
 * @param context   The context for which to set the stereo algorithm
 * @param algorithm The stereo algorithm in use by the provided context
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetStereoAlgorithm()
 */
enum TriclopsError triclopsGetStereoAlgorithm(const TriclopsContext context,
                                              TriclopsStereoAlgorithm *algorithm);

/** @} */ // End of StereoProcessing

/** @defgroup OCVParameters OpenCV stereo control functions
 *
 * Triclops integrates two OpenCV (OCV) stereo 3D reconstruction algorithms: Block Matching (BM) and Semi-Global Block
 * Matching (SGBM). The parameters for OCV stereo processing are separate from Triclops' Block Matching parameters,
 * allowing for a finer tuning of OCV algorithms, as all of the native parameters have been exposed through Triclops.
 *
 * Even though OCV stereo algorithms use the left frame as reference for stereo processing, Triclops has integrated
 * a frame processing pipeline that allows to use OCV stereo algorithms using the right frame as reference, making them
 * follow the same pattern as Triclops' Block Matching.
 *
 * Due to optimization constrains OCV stereo algorithms do not allow the computation of disparity ranges that are not
 * multiples of 16.
 *
 * # Speckle filter
 *
 * OpenCV runs a speckle removal filter at the end of each stereo computation, removing all the computation artifacts
 * based on two parameters:
 *
 *  * _Window size_ represents the maximum area a group of contiguous disparities should have to be considered an
 *                  artifact
 *  * _Maximum difference_ represents the maximum disparity difference between contiguous disparities to be included
 *                  in an artifact group. As OpenCV returns a fixed point disparity value map (equal to Triclops'
 *                  disparity map) the maximum disparity has to be provided in the same format (i.e. 12 integer bits
 *                  and 4 decimal bits).
 *
 * # Semi-Global Block Matching (SGBM) smoothness parameters
 *
 * SGMB uses two parameters in order to even out the resulting disparity map:
 *
 *  * The first parameter (P1) is the penalty on disparity change with a delta of one disparity between neighbouring
 *      pixels
 *  * The second parameter (P2) is the penalty on disparity change with a delta of more than one disparity between
 *      neighbouring pixels
 *
 *  The larger the parameters are the smoother and "flatter" the disparity map will result. As a rule of thumb a good
 *  smoothing parameters pair is: ```8*StereoMaskSize^2``` and ```32*StereoMaskSize^2```
 */

/** @addtogroup OCVParameters
 *
 * @{
 */  // start of group OCVParameters

/** @brief Sets a new Stereo Mask size for OpenCV Stereo Algorithms
 *
 * @param context           The context for which to set the Stereo Mask size
 * @param stereoMaskSize    The new stereo mask size, which needs to be a odd number between 5 and 255
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidSetting The provided stereo mask size is either not valid or is too large for the chosen
 *          output resolution
 *
 * @see triclopsGetOpenCVStereoMaskSize()
 */
enum TriclopsError triclopsSetOpenCVStereoMaskSize(const TriclopsContext context, int stereoMaskSize);

/** @brief Retrieves the current Stereo Mask size for OpenCV Stereo Algorithms
 *
 * @param context           The context from which to retrieve the Stereo Mask size
 * @param stereoMaskSize    The current stereo mask size
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetOpenCVStereoMaskSize()
 */
enum TriclopsError triclopsGetOpenCVStereoMaskSize(const TriclopsContext context, int *stereoMaskSize);

/** @brief Sets a new minimum disparity for OpenCV Stereo Algorithms
 *
 * @param context           The context for which to set the minimum disparity
 * @param minDisparity    The new minimum disparity
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorUnkown  The provided disparity generated an error into OpenCV. An extended error is returned
 *          via standard output
 *
 * @see triclopsGetOpenCVMinDisparity()
 * @see triclopsSetOpenCVNumDisparities()
 * @see triclopsSetOpenCVDisparityRange()
 */
enum TriclopsError triclopsSetOpenCVMinDisparity(const TriclopsContext context, int minDisparity);

/** @brief Retrieves the current minimum disparity for OpenCV Stereo Algorithms
 *
 * @param context           The context from which to get the minimum disparity
 * @param minDisparity    The current minimum disparity
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetOpenCVMinDisparity()
 * @see triclopsGetOpenCVNumDisparities()
 * @see triclopsGetOpenCVDisparityRange()
 */
enum TriclopsError triclopsGetOpenCVMinDisparity(const TriclopsContext context, int *minDisparity);

/** @brief Sets a new number of disparities for OpenCV Stereo Algorithms
 *
 * The number of disparities has to be a multiple of 16, due to OpenCV limitations.
 *
 * @param context           The context for which to set the number of disparities
 * @param numDisparities    The new number of disparities
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidSetting  The provided number of disparities is either a negative value or it is not a
 *          multiple of 16
 *
 * @see triclopsGetOpenCVNumDisparities()
 * @see triclopsSetOpenCVMinDisparity()
 * @see triclopsSetOpenCVDisparityRange()
 */
enum TriclopsError triclopsSetOpenCVNumDisparities(const TriclopsContext context, int numDisparities);

/** @brief Retrieves the current number of disparities for OpenCV Stereo Algorithms
 *
 * @param context           The context from which to get the number of disparities
 * @param numDisparities    The current number of disparities
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetOpenCVNumDisparities()
 * @see triclopsGetOpenCVMinDisparity()
 * @see triclopsGetOpenCVDisparityRange()
 */
enum TriclopsError triclopsGetOpenCVNumDisparities(const TriclopsContext context, int *numDisparities);

/** @brief Sets a new maximum disparity for OpenCV Stereo Algorithms
 *
 * This function is just syntactic sugar to replace
 *
 *  @code{.cpp}
 *  int maxDisparity, minDisparity;
 *  TriclopsContext context;
 *
 *  triclopsSetOpenCVNumDisparities(context, maxDisparity-minDisparity);
 *  @endcode
 *
 * The resulting number of disparities has to be a multiple of 16, due to OpenCV limitations.
 *
 * @param context           The context for which to set the maximum
 * @param maxDisparity    The new maximum disparity
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidSetting  The provided number of disparities is not a multiple of 16
 * @retval TriclopsErrorInvalidRequest The provided maximum is smaller than the current minimum
 *
 * @see triclopsGetOpenCVMaxDisparity()
 * @see triclopsSetOpenCVMinDisparity()
 * @see triclopsSetOpenCVNumDisparities()
 * @see triclopsSetOpenCVMinMaxDisparities()
 */
enum TriclopsError triclopsSetOpenCVMaxDisparity(const TriclopsContext context, int maxDisparity);

/** @brief Retrieves the current maximum disparity for OpenCV Stereo Algorithms
 *
 * @param context           The context from which to get the maximum disparity
 * @param maxDisparity    The current maximum disparity
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetOpenCVMaxDisparity()
 * @see triclopsGetOpenCVMinMaxDisparities()
 */
enum TriclopsError triclopsGetOpenCVMaxDisparity(const TriclopsContext context, int *maxDisparity);

/** @brief Sets new minimum and maximum disparities for OpenCV Stereo Algorithms
 *
 * The resulting number of disparities has to be a multiple of 16, due to OpenCV limitations.
 *
 * @param context           The context for which to set the minimum and maximum
 * @param minDisparity      The new minimum disparity
 * @param maxDisparity      The new maximum disparity
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidSetting  The resulting number of disparities is not a multiple of 16
 * @retval TriclopsErrorInvalidRequest The provided maximum is smaller than the provided minimum
 *
 * @see triclopsSetOpenCVMaxDisparity()
 * @see triclopsSetOpenCVMinDisparity()
 * @see triclopsSetOpenCVNumDisparities()
 * @see triclopsGetOpenCVMinMaxDisparities()
 */
enum TriclopsError triclopsSetOpenCVMinMaxDisparities(const TriclopsContext context, int minDisparity,
                                                      int maxDisparity);

/** @brief Retrieves the current minimum and maximum disparities for OpenCV Stereo Algorithms
 *
 * @param context           The context from which to get the minimum and maximum disparities
 * @param minDisparity      The current minimum disparity
 * @param maxDisparity      The current maximum disparity
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetOpenCVMinMaxDisparities()
 */
enum TriclopsError triclopsGetOpenCVMinMaxDisparities(const TriclopsContext context, int *minDisparity,
                                                      int *maxDisparity);

/** @brief Sets new minimum and number of disparities for OpenCV Stereo Algorithms
 *
 * The number of disparities has to be a multiple of 16, due to OpenCV limitations.
 *
 * @param context           The context for which to set the minimum and maximum
 * @param minDisparity      The new minimum disparity
 * @param numDisparities    The new number of disparities
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk              The function completed successfully
 * @retval TriclopsErrorInvalidContext  The provided context is invalid
 * @retval TriclopsErrorInvalidSetting  The number of disparities is not a multiple of 16
 * @retval TriclopsErrorUnknown         OpenCV returned an error while trying to set the provided minimum
 *
 * @see triclopsSetOpenCVMinDisparity()
 * @see triclopsSetOpenCVNumDisparities()
 * @see triclopsGetOpenCVDisparityRange()
 */
enum TriclopsError triclopsSetOpenCVDisparityRange(const TriclopsContext context, int minDisparity, int numDisparities);

/** @brief Retrieves the current minimum and maximum disparities for OpenCV Stereo Algorithms
 *
 * @param context           The context from which to get the minimum and maximum disparities
 * @param minDisparity      The current minimum disparity
 * @param numDisparities    The current number of disparities
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetOpenCVDisparityRange()
 */
enum TriclopsError triclopsGetOpenCVDisparityRange(const TriclopsContext context, int *minDisparity,
                                                   int *numDisparities);

/** @brief Sets a new speckle filter window size for OpenCV Stereo Algorithms
 *
 * @param context               The context for which to set the window size
 * @param speckleWindowSize     The new window size
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsGetOpenCVSpeckleWindowSize()
 */
enum TriclopsError triclopsSetOpenCVSpeckleWindowSize(const TriclopsContext context, int speckleWindowSize);

/** @brief Retrieves the current speckle filter window size for OpenCV Stereo Algorithms
 *
 * @param context               The context from which to get the window size
 * @param speckleWindowSize     The current window size
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetOpenCVSpeckleWindowSize()
 */
enum TriclopsError triclopsGetOpenCVSpeckleWindowSize(const TriclopsContext context, int *speckleWindowSize);

/** @brief Sets a new speckle filter maximum disparity for OpenCV Stereo Algorithms
 *
 * @param context               The context for which to set the maximum disparity
 * @param speckleRange          The new maximum disparity
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsGetOpenCVSpeckleRange()
 */
enum TriclopsError triclopsSetOpenCVSpeckleRange(const TriclopsContext context, int speckleRange);

/** @brief Retrieves the current speckle filter maximum disparity for OpenCV Stereo Algorithms
 *
 * @param context               The context from which to get the maximum disparity
 * @param speckleRange          The current maximum disparity
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetOpenCVSpeckleRange()
 */
enum TriclopsError triclopsGetOpenCVSpeckleRange(const TriclopsContext context, int *speckleRange);

/** @brief Sets a new maximum difference for OpenCV Stereo Algorithms' consistency checks
 *
 * @param context               The context for which to set the maximum difference in pixels
 * @param disp12MaxDiff         The new maximum difference in pixels
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsGetOpenCVDisp12MaxDiff()
 */
enum TriclopsError triclopsSetOpenCVDisp12MaxDiff(const TriclopsContext context, int disp12MaxDiff);

/** @brief Retrieves the current maximum difference for OpenCV Stereo Algorithms' consistency checks
 *
 * @param context               The context from which to get the maximum difference in pixels
 * @param disp12MaxDiff         The current maximum difference in pixels
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetOpenCVDisp12MaxDiff()
 */
enum TriclopsError triclopsGetOpenCVDisp12MaxDiff(const TriclopsContext context, int *disp12MaxDiff);

/** @brief Sets a new pre-filter for OpenCV Block Matching stereo algorithm
 *
 * @param context               The context for which to set the pre-filter
 * @param preFilterType         The new pre-filter
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidRequest The provided pre-filer is not supported or is invalid (e.g. casted integer)
 *
 * @see triclopsGetOpenCVBMPreFilterType()
 * @see triclopsSetStereoAlgorithm()
 * @see triclopsSetOpenCVBMPreFilterSize()
 */
enum TriclopsError triclopsSetOpenCVBMPreFilterType(const TriclopsContext context, OpenCVPrefilterType preFilterType);

/** @brief Retrieves the current pre-filter for OpenCV Block Matching stereo algorithm
 *
 * @param context               The context from which to get the pre-filter
 * @param prefilterType         The current pre-filter
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetOpenCVDisp12MaxDiff()
 */
enum TriclopsError triclopsGetOpenCVBMPreFilterType(const TriclopsContext context, OpenCVPrefilterType *prefilterType);

/** @brief Sets a new mask size for OpenCV Block Matching stereo algorithm Normalized response pre-filter
 *
 * This value only affects Normalized response pre-filter as X-Sobel pre-filter uses a fixed mask size.
 *
 * @param context               The context for which to set the mask size
 * @param preFilterSize         The new mask size (odd value between 5 and 255)
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidSetting The provided mask size is invalid
 *
 * @see triclopsGetOpenCVBMPreFilterSize()
 * @see triclopsSetOpenCVBMPreFilterType()
 * @see OpenCVPrefilterType
 */
enum TriclopsError triclopsSetOpenCVBMPreFilterSize(const TriclopsContext context, int preFilterSize);

/** @brief Retrieves the current mask size for OpenCV Block Matching stereo algorithm Normalized response pre-filter
 *
 * @param context               The context from which to get the mask size
 * @param preFilterSize         The current mask size
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetOpenCVBMPreFilterSize()
 */
enum TriclopsError triclopsGetOpenCVBMPreFilterSize(const TriclopsContext context, int *preFilterSize);

/** @brief Sets a new truncation value for OpenCV Block Matching stereo algorithm pre-filters
 *
 * @param context               The context for which to set the truncation value
 * @param preFilterCap          The new truncation value (value between 1 and 63)
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidSetting The provided truncation value is invalid
 *
 * @see triclopsGetOpenCVPreFilterCap()
 */
enum TriclopsError triclopsSetOpenCVPreFilterCap(const TriclopsContext context, int preFilterCap);

/** @brief Retrieves the current truncation value for OpenCV Block Matching stereo algorithm pre-filters
 *
 * @param context               The context from which to get the truncation value
 * @param preFilterCap          The current truncation value
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetOpenCVBMPreFilterSize()
 */
enum TriclopsError triclopsGetOpenCVPreFilterCap(const TriclopsContext context, int *preFilterCap);

/** @brief Sets a new texture threshold for OpenCV Block Matching stereo algorithm
 *
 * @param context               The context for which to set the truncation value
 * @param textureThreshold      The new texture threshold
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidSetting The provided texture threshold is invalid (i.e. negative)
 *
 * @see triclopsGetOpenCVBMTextureThreshold()
 */
enum TriclopsError triclopsSetOpenCVBMTextureThreshold(const TriclopsContext context, int textureThreshold);

/** @brief Retrieves the current texture threshold for OpenCV Block Matching stereo algorithm
 *
 * @param context               The context from which to get the truncation value
 * @param textureThreshold      The current texture threshold
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetOpenCVBMTextureThreshold()
 */
enum TriclopsError triclopsGetOpenCVBMTextureThreshold(const TriclopsContext context, int *textureThreshold);

/** @brief Sets a new uniqueness ratio for OpenCV stereo algorithms
 *
 * @param context               The context for which to set the uniqueness ratio
 * @param uniquenessRatio       The new uniqueness ratio
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidSetting The provided uniqueness ratio is invalid (i.e. negative)
 *
 * @see triclopsGetOpenCVBMTextureThreshold()
 */
enum TriclopsError triclopsSetOpenCVUniquenessRatio(const TriclopsContext context, int uniquenessRatio);

/** @brief Retrieves the current texture threshold for OpenCV stereo algorithms
 *
 * @param context               The context from which to get the uniqueness ratio
 * @param uniquenessRatio       The current uniqueness ratio
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetOpenCVBMTextureThreshold()
 */
enum TriclopsError triclopsGetOpenCVUniquenessRatio(const TriclopsContext context, int *uniquenessRatio);

/** @brief Sets a new first smoothing parameter for OpenCV Semi-Global Block Matching stereo algorithm
 *
 * @param context               The context for which to set the first smoothing parameter
 * @param P1                    The new first smoothing parameter
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsGetOpenCVSGBMP1()
 * @see triclopsSetOpenCVSGBMP2()
 * @see triclopsSetStereoAlgorithm()
 */
enum TriclopsError triclopsSetOpenCVSGBMP1(const TriclopsContext context, int P1);

/** @brief Retrieves the first smoothing parameter for OpenCV Semi-Global Block Matching stereo algorithm
 *
 * @param context               The context from which to get the first smoothing parameter
 * @param P1                    The current first smoothing parameter
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetOpenCVSGBMP1()
 * @see triclopsGetOpenCVSGBMP2()
 */
enum TriclopsError triclopsGetOpenCVSGBMP1(const TriclopsContext context, int *P1);

/** @brief Sets a new second smoothing parameter for OpenCV Semi-Global Block Matching stereo algorithm
 *
 * @param context               The context for which to set the second smoothing parameter
 * @param P2                    The new second smoothing parameter
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsGetOpenCVSGBMP1()
 * @see triclopsSetOpenCVSGBMP2()
 * @see triclopsSetStereoAlgorithm()
 */
enum TriclopsError triclopsSetOpenCVSGBMP2(const TriclopsContext context, int P2);

/** @brief Retrieves the second smoothing parameter for OpenCV Semi-Global Block Matching stereo algorithm
 *
 * @param context               The context from which to get the second smoothing parameter
 * @param P2                    The current second smoothing parameter
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetOpenCVSGBMP2()
 * @see triclopsGetOpenCVSGBMP1()
 */
enum TriclopsError triclopsGetOpenCVSGBMP2(const TriclopsContext context, int *P2);

/** @brief Sets a new computation mode for OpenCV Semi-Global Block Matching stereo algorithm
 *
 * @param context               The context for which to set the SGBM mode
 * @param mode                  The new SGBM mode
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidRequest The provided SGBM mode is not supported or is invalid (e.g. casted integer)
 *
 * @see triclopsGetOpenCVSGBMMode()
 * @see triclopsSetStereoAlgorithm()
 */
enum TriclopsError triclopsSetOpenCVSGBMMode(const TriclopsContext context, OpenCVSGBMMode mode);

/** @brief Retrieves the current computation mode for OpenCV Semi-Global Block Matching stereo algorithm
 *
 * @param context               The context from which to get the SGBM mode
 * @param mode                  The current SGBM mode
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetOpenCVSGBMMode()
 */
enum TriclopsError triclopsGetOpenCVSGBMMode(const TriclopsContext context, OpenCVSGBMMode *mode);

/** @} */ // End of OCVParameters

#ifdef __cplusplus
}
#endif

#endif  // #ifndef TRICLOPS_H
