//=============================================================================
// Copyright © 2016 FLIR Integrated Imaging Solutions, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of FLIR
// Integrated Imaging Solutions, Inc. ("Confidential Information"). You
// shall not disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with FLIR Integrated Imaging Solutions, Inc. (FLIR).
//
// FLIR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. FLIR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: triclopstype.h 309688 2016-12-21 18:28:11Z silvanoa $
//=============================================================================

//=============================================================================
/** @file triclopstype.h
 *  @brief Custom types */
//=============================================================================

#ifndef _TRICLOPS_TYPE_
#define _TRICLOPS_TYPE_

//=============================================================================
// Project Includes
//=============================================================================
#include "triclopsenum.h"

#ifdef __cplusplus
extern "C"
{
#endif

/** @defgroup TriclopsTypes Custom Types definitions */

/** @addtogroup TriclopsTypes
 *
 * @{ */ // start of group TriclopsTypes

/** @brief Boolean type definition kept for legacy purposes */
typedef int TriclopsBool;

/** @brief Wrapper for the internal C++ Triclops context
 *
 *  A Triclops context is the class that manages all the image buffers and bookkeeping information necessary
 *  to proper Triclops operation.
 */
typedef void *TriclopsContext;

/** @brief 8-bit greyscale image structure
 *
 *  Structure used in 8-bit greyscale image I/O operations. The rowinc parameter represents the length in bytes for each
 *  row; this value should be equal to ncols unless any padding is being performed. The buffer pointer, if
 *  changed, should point to an initialised memory location large at least rowinc*nrows bytes.
 */
typedef struct TriclopsImage {

    int nrows;             /**< The number of rows in the image, representing the image width */
    int ncols;             /**< The number of columns in the image, representing the image height */
    int rowinc;            /**< The number of bytes for each row */
    unsigned char *data = 0;   /**< Pointer to the location of the image buffer */
} TriclopsImage;

/** @brief 16-bit greyscale image structure
 *
 *  Structure used in 16-bit greyscale image I/O operations, most notably the retrieval of fixed point disparity maps.
 *  The rowinc parameter represents the length in bytes for each row; this value should be equal to
 *  ncols*sizeof(unsigned short) unless any padding is being performed. The buffer pointer, if
 *  changed, should point to an initialised memory location large at least rowinc*nrows bytes.
 *
 *  @see triclopsGetImage16()
 */
typedef struct TriclopsImage16 {
    int nrows;             /**< The number of rows in the image, representing the image width */
    int ncols;             /**< The number of columns in the image, representing the image height */
    int rowinc;            /**< The number of bytes for each row */
    unsigned short *data = 0;  /**< Pointer to the location of the image buffer */

} TriclopsImage16;


/** @brief Single colored pixel helper structure
 *
 * The structure has been defined in order to access color data more easily in colored images. As triclops uses 32-bit
 * RGBU color images the structure is defined to address four adjacent color values per pixel. 
 * Note: TriclopsColorPixel replaces the deprecated structure TriclopsPackedColorPixel defined in previous versions (< 4.0)
 * of Triclops.
 *
 * @see TriclopsColorImage
 */
typedef struct TriclopsColorPixel {
    unsigned char value[4]; /**< The RGBU color values packed in order (i.e. value[0] = R ... value[3] = U) */
} TriclopsColorPixel;

/** @brief 32-bit RGBU color image structure
 *
 *  Structure used in 32-bit color image I/O operations, most notably the retrieval of rectified colored inputs.
 *  The rowinc parameter represents the length in bytes for each row; this value should be equal to
 *  ncols*sizeof(struct TriclopsColorPixel) unless any padding is being performed. The buffer pointer, if
 *  changed, should point to an initialised memory location large at least rowinc*nrows bytes.
 *  Note : TriclopsColorImage replaces the deprecated image structure TriclopsPackedColorImage defined in previous 
 *  versions (<4.0) of Triclops.
 *
 * @see TriclopsColorPixel
 * @see triclopsGetColorImage()
 */
typedef struct TriclopsColorImage {
    int nrows;             /**< The number of rows in the image, representing the image width */
    int ncols;             /**< The number of columns in the image, representing the image height */
    int rowinc;            /**< The pitch or row increment of the image in terms of number of bytes
                                  between the beginning of a row and the beginning of the following row */
    TriclopsColorPixel *data = 0; /**< Pointer to the location of the image buffer */
} TriclopsColorImage;


/** @brief Greyscale image pair for stereo computation
 */
typedef struct TriclopsMonoStereoPair {
    TriclopsImage right; /**< Right or reference image for stereo computation */
    TriclopsImage left;  /**< Left image for stereo computation */

} TriclopsMonoStereoPair;


/** @brief Color image pair for stereo computation
 */
typedef struct TriclopsColorStereoPair {
    TriclopsColorImage right;    /**< Right or reference image for stereo computation */
    TriclopsColorImage left;     /**< Left image for stereo computation */

} TriclopsColorStereoPair;

/** @brief Region of Interest (ROI) structure
 *
 * Structure used to define ROIs for input image sub-processing. The image coordinates are computed with the origin
 * (0, 0) on the top-left corner of the image, with a maximum of (width, height) in the bottol-right corner of the image
 */
typedef struct TriclopsROI {
    int row;   /**< y-position of the upper left corner of the ROI with respect to the original image */
    int col;   /**< x-position of the upper left corner of the ROI with respect to the original image */
    int nrows; /**< Height of the ROI */
    int ncols; /**< Width in pixels of the ROI */

} TriclopsROI;

/** @brief Triclops SDK current version  
 *
 * This structure contains the current version of the Triclops library
 */
typedef struct TriclopsVersion {
    unsigned int major;  /**< Major version number. */
    unsigned int minor;  /**< Minor version number. */
    unsigned int type;   /**< Type version number. */
    unsigned int build;  /**< Build version number. */
} TriclopsVersion;

/*@}*/

#ifdef __cplusplus
}
#endif

#endif // _TRICLOPSTYPE_
