//=============================================================================
// Copyright © 2016 FLIR Integrated Imaging Solutions, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of FLIR
// Integrated Imaging Solutions, Inc. ("Confidential Information"). You
// shall not disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with FLIR Integrated Imaging Solutions, Inc. (FLIR).
//
// FLIR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. FLIR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: triclopsvalidation.h 309688 2016-12-21 18:28:11Z silvanoa $
//=============================================================================
//=============================================================================
/** @file triclopsvalidation.h
 *  @brief Stereo Validation API */
//=============================================================================
#ifndef TRICLOPSVALIDATION_H
#define TRICLOPSVALIDATION_H

//=============================================================================
// Project Includes
//=============================================================================
#include <triclops.h>

#ifdef __cplusplus
extern "C"
{
#endif

//=============================================================================
// Validation Support
//=============================================================================
//
// Group = Validation Support

/** @addtogroup Validation 
 *
 * @{ */


/** @brief Turns texture validation on or off
 *
 * Sets the context texture validation flag.  When set to true,
 * texture-based validation is enabled.  Pixels that do not pass
 * the validation test will be marked with the texture validation
 * mapping value in the disparity image.
 *
 * @param context The input context to set the texture validation flag for
 * @param on A Texture validation flag to set, indicating if texture validation 
 *           is enabled or disabled.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsGetTextureValidation()
 */
enum TriclopsError
triclopsSetTextureValidation(const TriclopsContext context,
                             TriclopsBool on);

/** @brief Retrieves the state of the texture validation flag
 *
 * @param context The input context.
 * @param on A pointer to the location for the retrieved texture validation flag.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetTextureValidation()
 */
enum TriclopsError
triclopsGetTextureValidation(const TriclopsContext context,
                             TriclopsBool *on);

/** @brief Sets the texture validation threshold
 *
 * Sets the texture validation threshold.  This threshold allows one
 *  to tune the texture-based rejection of pixels.  Values range from 
 *  0.0 (no rejection) to 128.0 (complete rejection) but good operating 
 *  range is between 0.0 and 2.0.
 *
 * @param context The context to set the texture validation threshold for
 * @param value The new texture validation threshold.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidSetting The requested threshold is out of range [0.0-128.0]
 *
 * @see triclopsGetTextureValidationThreshold()
 */
enum TriclopsError
triclopsSetTextureValidationThreshold(TriclopsContext context,
                                      float value);

/** @brief Retrieves the texture validation threshold
 *
 * @param context The context to retrieve the texture threshold value from
 * @param value  A pointer to the location for the retrieved texture threshold
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetTextureValidationThreshold()
 */
enum TriclopsError
triclopsGetTextureValidationThreshold(const TriclopsContext context,
                                      float *value);

/** @brief Sets the value that appears in the disparity image for pixels that
 *  fail texture validation
 *
 * @param context The context to set the texture validation mapping for.
 * @param value The new value to map invalid pixels to. The range is from 0 to 255.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidSetting The requested threshold is out of range [0-255]
 *
 * @see triclopsGetTextureValidationThreshold()
 */
enum TriclopsError
triclopsSetTextureValidationMapping(TriclopsContext context,
                                    unsigned char value);

/** @brief Gets the value that appears in the disparity image for pixels 
 *   that fail texture validation.
 *
 * @param context The context to retrieve the texture validation mapping value from
 * @param value  A pointer to the location for the retrieved mapping value
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetTextureValidationThreshold()
 */
enum TriclopsError
triclopsGetTextureValidationMapping(const TriclopsContext context,
                                    unsigned char *value);

/** @brief Enables or disables back-forth validation
 *
 * This function is used to enable or disable back-forth validation.
 *
 * @param context The input context to set the back-forth validation flag for
 * @param on A Boolean flag indicating if back-forth validation is enabled
 *            or disabled.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsGetBackForthValidation()
 */
enum TriclopsError
triclopsSetBackForthValidation(TriclopsContext context,
                               TriclopsBool on);
/** @brief Gets the current back-forth validation setting
 *
 * This function is used to obtain the current setting of back-forth validation.
 *
 * @param context The context to get the back-forth validation flag from
 * @param on  A pointer for the returned value of the current setting of the back-forth validation flag.
 * 
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetBackForthValidation()
 */
enum TriclopsError
triclopsGetBackForthValidation(const TriclopsContext context,
                               TriclopsBool *on);

/** @brief Sets the value that appears in the disparity image for pixels 
 *  that fail back-forth validation.
 *
 * Sets the back-forth validation mapping value.  This is the value that 
 * is assigned to pixels in the disparity image that fail the back-forth 
 * validation test.  The range is between 0 and 255.
 *
 * @param context The context to set the back-forth validation mapping for.
 * @param value The new value to map invalid pixels to. The range is from 0 to 255.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidSetting The requested threshold is out of range [0-255]
 *
 * @see triclopsGetBackForthValidationMapping()
 */
enum TriclopsError
triclopsSetBackForthValidationMapping(TriclopsContext context,
                                      unsigned char value);

/** @brief Gets the value that appears in the disparity image for pixels
*   that fail back-forth validation.
*
* @param context The context to retrieve the back-forth validation mapping value from
* @param value  A pointer to the location for the retrieved mapping value
*
* @return An error code representing the outcome of the function
* @retval TriclopsErrorOk The function completed successfully
* @retval TriclopsErrorInvalidContext The provided context is invalid
*
* @see triclopsGetBackForthValidationMapping()
*/
enum TriclopsError
triclopsGetBackForthValidationMapping(const TriclopsContext context,
                                      unsigned char *value);

/** @brief Enables or disables surface validation.
 *
 * This function is used to enable or disable surface validation.
 *
 * @param context The input context to set the surface validation flag for
 * @param on A Boolean flag indicating if surface validation is enabled
 *            or disabled.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsGetSurfaceValidation()
 */
enum TriclopsError
triclopsSetSurfaceValidation(TriclopsContext context,
                             TriclopsBool on);


/** @brief Gets the current surface validation setting
 *
 * This function is used to obtain the current setting of surface validation.
 *
 * @param context The context to get the surface validation flag from
 * @param on A pointer for the returned value of the current setting of the surface validation flag.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetSurfaceValidation()
 */
enum TriclopsError
triclopsGetSurfaceValidation(const TriclopsContext context,
                             TriclopsBool *on);


/** @brief Sets the minimum number of pixels a surface can cover and still be
 *  considered valid
 *
 * This function is used to set the minimum number of pixels a surface can
 * cover and still be considered valid.  The larger the number is, the fewer
 * surfaces will be accepted.  The lower the number is, the more surfaces will
 * be accepted.  Common parameter values range from 100 to 500, depending on
 * image resolution.
 *
 * @param context The context to set the surface validation size parameter for
 * @param size The minimum number of pixels a surface can cover and still be
 *             considered valid 
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidSetting A negative size was given.
 *
 * @see triclopsGetSurfaceValidationSize()
 */
enum TriclopsError
triclopsSetSurfaceValidationSize(TriclopsContext context,
                                 int size);

/** @brief Retrieves the current validation size.
 *
 * This function is used to extract the surface size parameter for surface
 * validation. 
 *
 * @param context The context to get the surface validation size from
 * @param size    A pointer to the location for the retrieved surface validation size
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetSurfaceValidationSize()
 */
enum TriclopsError
triclopsGetSurfaceValidationSize(const TriclopsContext context,
                                 int *size);

/** @brief Set the maximum disparity difference 
 *
 * Set the maximum disparity difference between two adjacent pixels that will
 * still allow the two pixels to be considered part of the same surface.
 *
 * @param context The context to set the surface validation size parameter for
 * @param diff The maximum disparity difference between two  adjacent pixels
 *            that will still allow the two pixels  to be considered part of
 *            the same surface.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidSetting A negative difference was given.
 *
 * @see triclopsGetSurfaceValidationDifference()
 */
enum TriclopsError
triclopsSetSurfaceValidationDifference(TriclopsContext context,
                                       float diff);

/** @brief Returns maximum disparity difference
 *
 * This function returns the maximum disparity difference between two adjacent pixels that
 * will still allow the two pixels to be considered part of the same surface.
 *
 * @param context The context to get the surface validation difference from
 * @param diff    A pointer to the location for the retrieved surface validation difference
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetSurfaceValidationSize()
 */
enum TriclopsError
triclopsGetSurfaceValidationDifference(const TriclopsContext context,
                                       float *diff);

/**@brief Sets the current surface validation mapping
 *
 * Surface validation is a noise rejection method.  By setting the mapping
 * value to 0x??, an invalid pixel that failed this validation step will be
 * marked 0xFF??. 
 *
 * @param context The context to set the surface validation mapping for.
 * @param value The new value to map invalid pixels to. The range is from 0 to 255.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidSetting The requested threshold is out of range [0-255]
 *
 * @see triclopsGetSurfaceValidationMapping()
 */
enum TriclopsError
triclopsSetSurfaceValidationMapping(TriclopsContext context,
                                    unsigned char value);

/** @brief Retrieves the current surface validation mapping.
 *
 * This function returns the current setting for the surface validation 
 * parameter.
 *
 * @param context The context to retrieve the surface validation mapping value from
 * @param value  A pointer to the location for the retrieved mapping value
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetSurfaceValidationMapping()
 */
enum TriclopsError
triclopsGetSurfaceValidationMapping(const TriclopsContext context,
                                    unsigned char *value);

/** @brief Sets the value that appears in the disparity image for pixels that fail 
 *  subpixel validation.
 *
 * Strict subpixel validation verifies that the disparity values contribute
 * to the subpixel interpolation make sense.  By setting the mapping value to
 * 0x??, an invalid pixel that failed this validation step will be marked
 * 0xFF??.
 *
 * @param context The context to set the subpixel validation mapping for.
 * @param value The new subpixel validation mapping value. The range is from 0 to 255.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 * @retval TriclopsErrorInvalidSetting The requested threshold is out of range [0-255]
 *
 * @see triclopsGetSubpixelValidationMapping()
 */
enum TriclopsError
triclopsSetSubpixelValidationMapping(TriclopsContext context,
                                     unsigned char value);

/** @brief Retrieves the value that appears in the disparity image for pixels that 
 *  fail subpixel validation.
 *
 * @param context The context to retrieve the subpixel validation mapping value from
 * @param value  A pointer to the location for the retrieved mapping value
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk The function completed successfully
 * @retval TriclopsErrorInvalidContext The provided context is invalid
 *
 * @see triclopsSetSubpixelValidationMapping()
 */
enum TriclopsError
triclopsGetSubpixelValidationMapping(const TriclopsContext context,
                                     unsigned char *value);

/*@}*/ // End of group Validation

#ifdef __cplusplus
}
#endif

#endif  // #ifndef TRICLOPSVALIDATION_H
